"""CLI for mem0-open-mcp using Typer."""

from __future__ import annotations

import json
import logging
import logging.handlers
import time
from datetime import datetime
from pathlib import Path
from typing import Annotated

logger = logging.getLogger(__name__)

import typer
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm, Prompt
from rich.table import Table

from mem0_server import __version__
from mem0_server.config import (
    ConfigLoader,
    EmbedderProviderType,
    LLMProviderType,
    Mem0ServerConfig,
    VectorStoreProviderType,
)

app = typer.Typer(
    name="mem0-open-mcp",
    help="Standalone MCP server for mem0 with web configuration UI.",
    add_completion=False,
    rich_markup_mode="rich",
)

console = Console()

UPDATE_CHECK_CACHE = Path.home() / ".cache" / "mem0-open-mcp" / "update_check.json"
UPDATE_CHECK_INTERVAL = 86400


def _setup_file_logging(log_level: str = "info", mode: str = "server") -> Path | None:
    from mem0_server.config.loader import MEM0_LOGS_DIR

    MEM0_LOGS_DIR.mkdir(parents=True, exist_ok=True)

    log_file = MEM0_LOGS_DIR / f"{mode}.log"

    root_logger = logging.getLogger()
    root_logger.setLevel(getattr(logging, log_level.upper()))

    file_handler = logging.handlers.RotatingFileHandler(
        log_file,
        maxBytes=10 * 1024 * 1024,
        backupCount=5,
        encoding="utf-8",
    )
    file_handler.setFormatter(
        logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    )
    root_logger.addHandler(file_handler)

    perf_logger = logging.getLogger("mem0_server.performance")
    perf_file = MEM0_LOGS_DIR / "performance.log"
    perf_handler = logging.handlers.RotatingFileHandler(
        perf_file,
        maxBytes=10 * 1024 * 1024,
        backupCount=5,
        encoding="utf-8",
    )
    perf_handler.setFormatter(logging.Formatter("%(asctime)s  %(message)s"))
    perf_logger.addHandler(perf_handler)
    perf_logger.setLevel(logging.INFO)
    perf_logger.propagate = False

    return log_file


def _parse_version(v: str) -> tuple[int, ...]:
    """Parse version string to tuple for comparison."""
    try:
        return tuple(int(x) for x in v.split(".")[:3])
    except (ValueError, AttributeError):
        return (0, 0, 0)


def _check_for_updates() -> tuple[str | None, bool]:
    """Check PyPI for newer version (once per day).

    Returns:
            (latest_version, is_update_available)
    """
    try:
        UPDATE_CHECK_CACHE.parent.mkdir(parents=True, exist_ok=True)

        now = time.time()
        if UPDATE_CHECK_CACHE.exists():
            cache = json.loads(UPDATE_CHECK_CACHE.read_text())
            if now - cache.get("last_check", 0) < UPDATE_CHECK_INTERVAL:
                latest = cache.get("latest")
                if latest and _parse_version(latest) > _parse_version(__version__):
                    return (latest, True)
                return (latest, False)

        import httpx

        resp = httpx.get("https://pypi.org/pypi/mem0-open-mcp/json", timeout=3)
        resp.raise_for_status()
        latest = resp.json()["info"]["version"]

        UPDATE_CHECK_CACHE.write_text(json.dumps({"last_check": now, "latest": latest}))

        if _parse_version(latest) > _parse_version(__version__):
            return (latest, True)
        return (latest, False)
    except Exception:
        return (None, False)


def _model_matches(config_model: str, available_models: list[str]) -> bool:
    """Check if configured model matches any available model.

    Handles :latest suffix - 'llama3.2' matches 'llama3.2:latest'
    """
    config_lower = config_model.lower()
    config_with_latest = f"{config_lower}:latest" if ":" not in config_lower else config_lower

    for m in available_models:
        m_lower = m.lower()
        if config_lower == m_lower:
            return True
        if config_with_latest == m_lower:
            return True
        m_without_latest = (
            m_lower.replace(":latest", "") if m_lower.endswith(":latest") else m_lower
        )
        if config_lower == m_without_latest:
            return True
    return False


def _print_timing_summary(timings: dict[str, float], config: "Mem0ServerConfig") -> None:
    """Print a summary table of operation timings."""
    from rich.table import Table

    table = Table(
        title="[bold]Response Time Summary[/bold]", show_header=True, header_style="bold cyan"
    )
    table.add_column("Operation", style="dim", width=20)
    table.add_column("Time (s)", justify="right", width=12)
    table.add_column("Components Involved", width=40)

    # Component info
    llm_info = f"LLM ({config.llm.provider.value})"
    embedder_info = f"Embedder ({config.embedder.provider.value})"
    vector_store_info = f"VectorStore ({config.vector_store.provider.value})"
    graph_info = f"GraphStore ({config.graph_store.provider.value})" if config.graph_store else None

    # Define what each operation involves
    op_components = {
        "init": "Client initialization",
        "add": f"{llm_info}, {embedder_info}, {vector_store_info}"
        + (f", {graph_info}" if graph_info else ""),
        "list": vector_store_info,
        "search": f"{embedder_info}, {vector_store_info}",
        "delete": vector_store_info,
        "cleanup": vector_store_info,
    }

    op_names = {
        "init": "Initialize",
        "add": "Add Memory",
        "list": "List Memories",
        "search": "Search Memories",
        "delete": "Delete Memory",
        "cleanup": "Cleanup All",
    }

    total_time = 0.0
    for op_key in ["init", "add", "list", "search", "delete", "cleanup"]:
        if op_key in timings:
            time_val = timings[op_key]
            total_time += time_val
            time_style = "red" if time_val > 5.0 else "yellow" if time_val > 2.0 else "green"
            table.add_row(
                op_names.get(op_key, op_key),
                f"[{time_style}]{time_val:.2f}[/{time_style}]",
                op_components.get(op_key, ""),
            )

    table.add_row("", "", "")
    table.add_row("[bold]Total[/bold]", f"[bold]{total_time:.2f}[/bold]", "")

    console.print(table)
    console.print()

    # Additional breakdown hint
    console.print(
        "[dim]💡 Tip: 'Add Memory' includes LLM processing (memory extraction) + embedding + storage[/dim]"
    )
    console.print("[dim]   'Search' includes embedding query + vector similarity search[/dim]")
    console.print()


def _run_connectivity_tests(config: Mem0ServerConfig) -> bool:
    """Run connectivity tests for LLM, Embedder, and Vector Store."""
    console.print("[bold]Running connectivity tests...[/bold]\n")

    all_passed = True

    # Test Vector Store
    console.print("  [dim]Vector Store...[/dim]", end=" ")
    try:
        vs_config = config.vector_store
        if vs_config.provider.value == "qdrant":
            from qdrant_client import QdrantClient

            host = vs_config.config.host or "localhost"
            port = vs_config.config.port or 6333
            client = QdrantClient(host=host, port=port, timeout=5)
            client.get_collections()
            console.print("[green]✓ Connected[/green]")
        elif vs_config.provider.value == "chroma":
            import chromadb

            if vs_config.config.host:
                client = chromadb.HttpClient(
                    host=vs_config.config.host, port=vs_config.config.port or 8000
                )
            else:
                client = chromadb.Client()
            client.heartbeat()
            console.print("[green]✓ Connected[/green]")
        else:
            console.print(f"[yellow]⚠ Skip (no test for {vs_config.provider.value})[/yellow]")
    except Exception as e:
        console.print(f"[red]✗ Failed: {e}[/red]")
        all_passed = False

    # Test LLM
    console.print("  [dim]LLM...[/dim]", end=" ")
    try:
        llm_config = config.llm
        if llm_config.provider.value == "ollama":
            import httpx

            base_url = llm_config.config.base_url or "http://localhost:11434"
            resp = httpx.get(f"{base_url}/api/tags", timeout=5)
            resp.raise_for_status()
            models = [m["name"] for m in resp.json().get("models", [])]
            if _model_matches(llm_config.config.model, models):
                console.print(f"[green]✓ Connected ({llm_config.config.model})[/green]")
            else:
                console.print(
                    f"[yellow]⚠ Connected but model '{llm_config.config.model}' not found[/yellow]"
                )
                console.print(
                    f"      [dim]Available: {', '.join(models[:5])}{'...' if len(models) > 5 else ''}[/dim]"
                )
        elif llm_config.provider.value in ("openai", "lmstudio"):
            import httpx

            base_url = llm_config.config.base_url or "https://api.openai.com/v1"
            headers = {}
            if llm_config.config.api_key:
                headers["Authorization"] = f"Bearer {llm_config.config.api_key}"
            resp = httpx.get(f"{base_url}/models", headers=headers, timeout=5)
            resp.raise_for_status()
            console.print(f"[green]✓ Connected ({llm_config.config.model})[/green]")
        else:
            console.print(f"[yellow]⚠ Skip (no test for {llm_config.provider.value})[/yellow]")
    except Exception as e:
        console.print(f"[red]✗ Failed: {e}[/red]")
        all_passed = False

    # Test Embedder
    console.print("  [dim]Embedder...[/dim]", end=" ")
    try:
        emb_config = config.embedder
        if emb_config.provider.value == "ollama":
            import httpx

            base_url = emb_config.config.base_url or "http://localhost:11434"
            resp = httpx.get(f"{base_url}/api/tags", timeout=5)
            resp.raise_for_status()
            models = [m["name"] for m in resp.json().get("models", [])]
            if _model_matches(emb_config.config.model, models):
                console.print(f"[green]✓ Connected ({emb_config.config.model})[/green]")
            else:
                console.print(
                    f"[yellow]⚠ Connected but model '{emb_config.config.model}' not found[/yellow]"
                )
                console.print(
                    f"      [dim]Available: {', '.join(models[:5])}{'...' if len(models) > 5 else ''}[/dim]"
                )
        elif emb_config.provider.value in ("openai", "lmstudio"):
            import httpx

            base_url = emb_config.config.base_url or "https://api.openai.com/v1"
            headers = {}
            if emb_config.config.api_key:
                headers["Authorization"] = f"Bearer {emb_config.config.api_key}"
            resp = httpx.get(f"{base_url}/models", headers=headers, timeout=5)
            resp.raise_for_status()
            console.print(f"[green]✓ Connected ({emb_config.config.model})[/green]")
        else:
            console.print(f"[yellow]⚠ Skip (no test for {emb_config.provider.value})[/yellow]")
    except Exception as e:
        console.print(f"[red]✗ Failed: {e}[/red]")
        all_passed = False

    # Test Graph Store (if configured)
    if config.graph_store:
        console.print("  [dim]Graph Store...[/dim]", end=" ")
        try:
            gs_config = config.graph_store
            if gs_config.provider.value == "neo4j":
                from langchain_neo4j import Neo4jGraph

                g = Neo4jGraph(
                    url=gs_config.config.url,
                    username=gs_config.config.username,
                    password=gs_config.config.password,
                    database=gs_config.config.database,
                )
                g.query("RETURN 1")
                console.print(f"[green]✓ Connected ({gs_config.provider.value})[/green]")
            elif gs_config.provider.value == "kuzu":
                console.print(f"[green]✓ Configured ({gs_config.provider.value})[/green]")
            else:
                console.print(f"[yellow]⚠ Skip (no test for {gs_config.provider.value})[/yellow]")
        except Exception as e:
            console.print(f"[red]✗ Failed: {e}[/red]")
            all_passed = False

    console.print()
    if all_passed:
        console.print("[bold green]All connectivity tests passed![/bold green]\n")
    else:
        console.print(
            "[bold red]Some connectivity tests failed. Please check your configuration.[/bold red]\n"
        )

    return all_passed


def _run_memory_tests(config: Mem0ServerConfig, custom_message: str | None = None) -> bool:
    """Run actual mem0 memory add/search tests."""
    import asyncio

    return asyncio.run(
        _run_memory_tests_async_impl(
            config,
            custom_message,
            header="[bold]Running memory tests...[/bold]\n",
            init_label="Initializing mem0 client",
            success_label="All memory tests passed!",
            failure_label="Memory test failed. Check your LLM/Embedder/VectorStore configuration.",
        )
    )


def _run_memory_tests_async(config: Mem0ServerConfig, custom_message: str | None = None) -> bool:
    """Run mem0 memory tests using AsyncMemory."""
    import asyncio

    return asyncio.run(
        _run_memory_tests_async_impl(
            config,
            custom_message,
            header="[bold]Running async memory tests...[/bold]\n",
            init_label="Initializing AsyncMemory client",
            success_label="All async memory tests passed!",
            failure_label="Async memory test failed. Check your LLM/Embedder/VectorStore configuration.",
        )
    )


async def _run_memory_tests_async_impl(
    config: Mem0ServerConfig,
    custom_message: str | None = None,
    header: str = "[bold]Running async memory tests...[/bold]\n",
    init_label: str = "Initializing AsyncMemory client",
    success_label: str = "All async memory tests passed!",
    failure_label: str = "Async memory test failed. Check your LLM/Embedder/VectorStore configuration.",
) -> bool:
    """Async implementation of memory tests."""
    import asyncio
    import os
    import sys
    import time
    import uuid

    from mem0_server.memory_ops import (
        add_memory_op,
        delete_all_memories_op,
        delete_memory_op,
        list_memories_op,
        search_memory_op,
    )

    # Setup performance logging if enabled
    perf_handler = None
    if config.server.performance_logging:
        perf_logger = logging.getLogger("mem0_server.performance")
        perf_logger.setLevel(logging.INFO)
        perf_handler = logging.StreamHandler(sys.stderr)
        perf_handler.setFormatter(logging.Formatter("%(message)s"))
        perf_logger.addHandler(perf_handler)

    console.print(header)

    test_user_id = f"__test_user_{uuid.uuid4().hex[:8]}"
    default_message = "Alice is a software engineer at TechCorp. She knows Bob who is a data scientist at DataLab. They collaborate on AI projects."
    test_memory_text = custom_message if custom_message else default_message
    max_retries = 20
    retry_interval = 0.5

    # Timing results storage
    timings: dict[str, float] = {}

    console.print(f"  [dim]Test message:[/dim]")
    console.print(f'    [italic]"{test_memory_text}"[/italic]\n')

    try:
        console.print(f"  [dim]{init_label}...[/dim]", end=" ")
        init_start = time.perf_counter()
        os.environ["MEM0_TELEMETRY"] = "false"
        from mem0 import AsyncMemory

        mem0_config = config.to_mem0_config()
        memory = await AsyncMemory.from_config(mem0_config)
        timings["init"] = time.perf_counter() - init_start
        console.print(f"[green]✓[/green] [dim]({timings['init']:.2f}s)[/dim]")

        # 1. Add memory (involves LLM + Embedder + Vector Store)
        console.print("  [dim]1. Adding test memory...[/dim]", end=" ")
        add_start = time.perf_counter()
        add_result = await add_memory_op(
            memory_client=memory,
            text=test_memory_text,
            user_id=test_user_id,
            metadata=None,
            config=config,
        )
        timings["add"] = time.perf_counter() - add_start
        memory_id = None
        if add_result and add_result.get("results") and len(add_result["results"]) > 0:
            first_result = add_result["results"][0]
            memory_id = first_result.get("id") if first_result else None
            if memory_id:
                console.print(
                    f"[green]✓ Added (id: {memory_id[:8]}...)[/green] [dim]({timings['add']:.2f}s)[/dim]"
                )
            else:
                console.print(
                    f"[yellow]⚠ No memory extracted by LLM[/yellow] [dim]({timings['add']:.2f}s)[/dim]"
                )
        else:
            console.print(
                f"[yellow]⚠ No memory extracted by LLM[/yellow] [dim]({timings['add']:.2f}s)[/dim]"
            )
            console.print(f"    [dim]add_result: {add_result}[/dim]")

        # 2. List memories (with retry) - involves Vector Store
        console.print("  [dim]2. Listing memories...[/dim]", end=" ")
        list_start = time.perf_counter()
        stored_count = 0
        list_result = None
        for attempt in range(max_retries):
            list_result = await list_memories_op(
                memory_client=memory,
                user_id=test_user_id,
                config=config,
            )
            if list_result and list_result.get("results"):
                stored_count = len(list_result["results"])
                if not memory_id and stored_count > 0:
                    memory_id = list_result["results"][0].get("id")
                break
            if attempt < max_retries - 1:
                await asyncio.sleep(retry_interval)
        timings["list"] = time.perf_counter() - list_start

        if stored_count > 0:
            console.print(
                f"[green]✓ Found {stored_count} memory(s)[/green] [dim]({timings['list']:.2f}s)[/dim]"
            )
            # Show extracted memories
            if list_result and list_result.get("results"):
                for i, mem in enumerate(list_result["results"][:5]):
                    mem_text = mem.get("memory", "")[:50]
                    console.print(f"       [dim]{i + 1}. {mem_text}...[/dim]")
        else:
            console.print(
                f"[red]✗ Memory not stored (waited 10s)[/red] [dim]({timings['list']:.2f}s)[/dim]"
            )
            return False

        if hasattr(memory, "enable_graph") and memory.enable_graph:
            console.print("  [dim]   Graph store...[/dim]", end=" ")
            if list_result and list_result.get("relations") is not None:
                relations = list_result.get("relations", [])
                relations_count = len(relations)
                console.print(f"[green]✓ Enabled ({relations_count} relation(s))[/green]")
                # Show extracted relations
                for rel in relations[:5]:
                    src = rel.get("source", "?")
                    r = rel.get("relationship", "?")
                    tgt = rel.get("target", rel.get("destination", "?"))
                    console.print(f"       [dim]• {src} --[{r}]--> {tgt}[/dim]")
            else:
                console.print("[yellow]⚠ Enabled but no relations returned[/yellow]")

        # 3. Search memories - involves Embedder + Vector Store
        console.print("  [dim]3. Searching memories...[/dim]", end=" ")
        search_start = time.perf_counter()
        search_result = await search_memory_op(
            memory_client=memory,
            query="test memory verification",
            user_id=test_user_id,
            limit=5,
            config=config,
        )
        timings["search"] = time.perf_counter() - search_start
        if search_result and search_result.get("results"):
            console.print(
                f"[green]✓ Found {len(search_result['results'])} result(s)[/green] [dim]({timings['search']:.2f}s)[/dim]"
            )
        else:
            console.print(
                f"[yellow]⚠ No search results (indexing may be delayed)[/yellow] [dim]({timings['search']:.2f}s)[/dim]"
            )

        # 4. Delete single memory (with retry verification) - involves Vector Store
        console.print("  [dim]4. Deleting single memory...[/dim]", end=" ")
        delete_start = time.perf_counter()
        if memory_id:
            await delete_memory_op(
                memory_client=memory,
                memory_id=memory_id,
                config=config,
            )
            deleted = False
            for attempt in range(max_retries):
                list_result = await list_memories_op(
                    memory_client=memory,
                    user_id=test_user_id,
                    config=config,
                )
                results = list_result.get("results", []) if list_result else []
                if not any(m.get("id") == memory_id for m in results):
                    deleted = True
                    break
                if attempt < max_retries - 1:
                    await asyncio.sleep(retry_interval)
            timings["delete"] = time.perf_counter() - delete_start

            if deleted:
                console.print(
                    f"[green]✓ Deleted (id: {memory_id[:8]}...)[/green] [dim]({timings['delete']:.2f}s)[/dim]"
                )
            else:
                console.print(
                    f"[yellow]⚠ Delete called but not confirmed after 10s[/yellow] [dim]({timings['delete']:.2f}s)[/dim]"
                )
        else:
            timings["delete"] = time.perf_counter() - delete_start
            console.print(
                f"[yellow]⚠ Skipped (no memory_id)[/yellow] [dim]({timings['delete']:.2f}s)[/dim]"
            )

        # 5. Cleanup remaining test data (with retry verification) - involves Vector Store
        console.print("  [dim]5. Cleaning up test data...[/dim]", end=" ")
        cleanup_start = time.perf_counter()
        deleted_count = await delete_all_memories_op(
            memory_client=memory,
            user_id=test_user_id,
            config=config,
            use_individual_delete=True,
        )
        cleaned = False
        for attempt in range(max_retries):
            list_result = await list_memories_op(
                memory_client=memory,
                user_id=test_user_id,
                config=config,
            )
            results = list_result.get("results", []) if list_result else []
            if len(results) == 0:
                cleaned = True
                break
            if attempt < max_retries - 1:
                await asyncio.sleep(retry_interval)
        timings["cleanup"] = time.perf_counter() - cleanup_start

        if cleaned:
            console.print(f"[green]✓ Cleaned[/green] [dim]({timings['cleanup']:.2f}s)[/dim]")
        else:
            console.print(
                f"[yellow]⚠ Cleanup called but not confirmed after 10s[/yellow] [dim]({timings['cleanup']:.2f}s)[/dim]"
            )

        console.print()

        # Print timing summary
        _print_timing_summary(timings, config)

        console.print(f"[bold green]{success_label}[/bold green]\n")

        # Cleanup perf handler
        if perf_handler:
            perf_logger.removeHandler(perf_handler)

        return True

    except Exception as e:
        console.print(f"[red]✗ Failed: {e}[/red]")
        console.print()
        console.print(f"[bold red]{failure_label}[/bold red]\n")

        # Cleanup perf handler
        if perf_handler:
            perf_logger.removeHandler(perf_handler)

        return False


def _run_profiling_tests(config: Mem0ServerConfig, custom_message: str | None = None) -> bool:
    """Run detailed profiling of each component in the add memory operation."""
    import os
    import time
    import json

    console.print("[bold]Running component profiling...[/bold]\n")

    default_message = "Alice is a software engineer at TechCorp. She knows Bob who is a data scientist at DataLab. They collaborate on AI projects."
    test_memory_text = custom_message if custom_message else default_message

    console.print(f"  [dim]Test message:[/dim]")
    console.print(f'    [italic]"{test_memory_text}"[/italic]\n')

    timings: dict[str, float] = {}

    try:
        os.environ["MEM0_TELEMETRY"] = "false"

        # 1. Initialize Memory client and access internal components
        console.print("  [dim]1. Initializing components...[/dim]", end=" ")
        init_start = time.perf_counter()

        from mem0 import Memory
        from mem0.memory.utils import get_fact_retrieval_messages, parse_messages

        mem0_config = config.to_mem0_config()
        memory = Memory.from_config(mem0_config)

        timings["init"] = time.perf_counter() - init_start
        console.print(f"[green]✓[/green] [dim]({timings['init']:.2f}s)[/dim]")

        # Get internal components
        llm = memory.llm
        embedding_model = memory.embedding_model
        vector_store = memory.vector_store

        # 2. LLM Fact Extraction
        console.print("  [dim]2. LLM Fact Extraction...[/dim]", end=" ")
        llm_start = time.perf_counter()

        messages = [{"role": "user", "content": test_memory_text}]
        parsed_messages = parse_messages(messages)

        # Use custom prompt if configured, otherwise use default
        if config.custom_prompts.fact_extraction:
            system_prompt = config.custom_prompts.fact_extraction
            user_prompt = f"Input:\n{parsed_messages}"
        else:
            system_prompt, user_prompt = get_fact_retrieval_messages(parsed_messages, False)

        response = llm.generate_response(
            messages=[
                {"role": "system", "content": system_prompt},
                {"role": "user", "content": user_prompt},
            ],
            response_format={"type": "json_object"},
        )

        timings["llm_extraction"] = time.perf_counter() - llm_start

        # Parse facts
        try:
            from mem0.memory.utils import remove_code_blocks

            response = remove_code_blocks(response)
            facts = json.loads(response).get("facts", [])
        except Exception:
            facts = []

        console.print(
            f"[green]✓ Extracted {len(facts)} fact(s)[/green] [dim]({timings['llm_extraction']:.2f}s)[/dim]"
        )
        for i, fact in enumerate(facts[:5]):
            console.print(
                f"       [dim]{i + 1}. {fact[:60]}{'...' if len(fact) > 60 else ''}[/dim]"
            )

        # 3. Embedding Generation
        console.print("  [dim]3. Embedding Generation...[/dim]", end=" ")
        embed_start = time.perf_counter()

        embeddings = []
        for fact in facts:
            emb = embedding_model.embed(fact, "add")
            embeddings.append(emb)

        timings["embedding"] = time.perf_counter() - embed_start
        embed_dim = len(embeddings[0]) if embeddings else 0
        console.print(
            f"[green]✓ {len(embeddings)} embedding(s), dim={embed_dim}[/green] [dim]({timings['embedding']:.2f}s)[/dim]"
        )

        # 4. Vector Store Search (for existing memories)
        console.print("  [dim]4. Vector Store Search...[/dim]", end=" ")
        search_start = time.perf_counter()

        search_results = []
        for i, (fact, emb) in enumerate(zip(facts, embeddings)):
            results = vector_store.search(
                query=fact,
                vectors=emb,
                limit=5,
                filters={},
            )
            search_results.append(len(results))

        timings["vector_search"] = time.perf_counter() - search_start
        total_found = sum(search_results)
        console.print(
            f"[green]✓ Found {total_found} existing match(es)[/green] [dim]({timings['vector_search']:.2f}s)[/dim]"
        )

        # 5. LLM Memory Update Decision
        if facts:
            console.print("  [dim]5. LLM Update Decision...[/dim]", end=" ")
            update_start = time.perf_counter()

            from mem0.configs.prompts import get_update_memory_messages

            # Simulate update decision call
            old_memories = []  # No existing memories for profiling
            function_calling_prompt = get_update_memory_messages(old_memories, facts, None)

            # The function returns a string prompt, not a list of messages
            update_response = llm.generate_response(
                messages=[
                    {"role": "system", "content": "You are a memory management assistant."},
                    {"role": "user", "content": function_calling_prompt},
                ],
                response_format={"type": "json_object"},
            )
            _ = update_response

            timings["llm_update"] = time.perf_counter() - update_start
            console.print(f"[green]✓[/green] [dim]({timings['llm_update']:.2f}s)[/dim]")
        else:
            timings["llm_update"] = 0.0
            console.print(
                "  [dim]5. LLM Update Decision...[/dim] [yellow]⚠ Skipped (no facts)[/yellow]"
            )

        # 6. Vector Store Insert (estimated based on search latency)
        # Note: We skip actual insert to avoid creating test data
        # Insert latency is typically similar to search latency
        console.print("  [dim]6. Vector Store Insert...[/dim]", end=" ")
        # Estimate based on search time (insert is typically similar or slightly slower)
        timings["vector_insert"] = (
            timings["vector_search"] * 1.2 if timings["vector_search"] > 0 else 0.01
        )
        console.print(f"[cyan]~ estimated[/cyan] [dim]({timings['vector_insert']:.2f}s)[/dim]")

        # 7. Graph Store (if enabled)
        if hasattr(memory, "enable_graph") and memory.enable_graph and hasattr(memory, "graph"):
            console.print("  [dim]7. Graph Store...[/dim]", end=" ")
            graph_start = time.perf_counter()

            # Just measure a simple graph query
            try:
                if hasattr(memory.graph, "search"):
                    memory.graph.search("test query", {}, limit=1)
                timings["graph_store"] = time.perf_counter() - graph_start
                console.print(f"[green]✓[/green] [dim]({timings['graph_store']:.2f}s)[/dim]")
            except Exception as e:
                timings["graph_store"] = time.perf_counter() - graph_start
                console.print(f"[yellow]⚠ {e}[/yellow] [dim]({timings['graph_store']:.2f}s)[/dim]")
        else:
            timings["graph_store"] = 0.0

        console.print()

        # Print detailed profiling summary
        _print_profiling_summary(timings, config)

        return True

    except Exception as e:
        import traceback

        console.print(f"[red]✗ Failed: {e}[/red]")
        console.print(f"[dim]{traceback.format_exc()}[/dim]")
        return False


def _print_profiling_summary(timings: dict[str, float], config: "Mem0ServerConfig") -> None:
    """Print detailed profiling summary with breakdown of add memory operation."""
    from rich.table import Table

    table = Table(
        title="[bold]Add Memory Operation Breakdown[/bold]",
        show_header=True,
        header_style="bold cyan",
    )
    table.add_column("Step", style="dim", width=30)
    table.add_column("Component", width=20)
    table.add_column("Time (s)", justify="right", width=12)
    table.add_column("% of Total", justify="right", width=12)

    # Calculate total (excluding init)
    operation_timings = {k: v for k, v in timings.items() if k != "init"}
    total_op_time = sum(operation_timings.values())

    # Component mapping
    components = {
        "llm_extraction": f"LLM ({config.llm.provider.value})",
        "embedding": f"Embedder ({config.embedder.provider.value})",
        "vector_search": f"VectorStore ({config.vector_store.provider.value})",
        "llm_update": f"LLM ({config.llm.provider.value})",
        "vector_insert": f"VectorStore ({config.vector_store.provider.value})",
        "graph_store": f"GraphStore ({config.graph_store.provider.value})"
        if config.graph_store
        else "N/A",
    }

    step_names = {
        "llm_extraction": "1. Fact Extraction",
        "embedding": "2. Generate Embeddings",
        "vector_search": "3. Search Existing Memories",
        "llm_update": "4. Memory Update Decision",
        "vector_insert": "5. Insert New Memories",
        "graph_store": "6. Graph Relations",
    }

    # Add init row first
    if "init" in timings:
        init_time = timings["init"]
        table.add_row(
            "0. Initialize Client", "All components", f"{init_time:.2f}", "[dim]N/A[/dim]"
        )
        table.add_row("", "", "", "")

    # Add operation rows
    for key in [
        "llm_extraction",
        "embedding",
        "vector_search",
        "llm_update",
        "vector_insert",
        "graph_store",
    ]:
        if key in timings and timings[key] > 0:
            time_val = timings[key]
            pct = (time_val / total_op_time * 100) if total_op_time > 0 else 0

            # Color coding
            time_style = "red" if time_val > 5.0 else "yellow" if time_val > 2.0 else "green"
            pct_style = "red" if pct > 50 else "yellow" if pct > 25 else "white"

            table.add_row(
                step_names.get(key, key),
                components.get(key, "Unknown"),
                f"[{time_style}]{time_val:.2f}[/{time_style}]",
                f"[{pct_style}]{pct:.1f}%[/{pct_style}]",
            )

    # Total row
    table.add_row("", "", "", "")
    table.add_row(
        "[bold]Total Operation Time[/bold]",
        "",
        f"[bold]{total_op_time:.2f}[/bold]",
        "[bold]100%[/bold]",
    )

    console.print(table)
    console.print()

    # Bottleneck analysis
    if operation_timings:
        bottleneck_key = max(operation_timings, key=operation_timings.get)
        bottleneck_time = operation_timings[bottleneck_key]
        bottleneck_pct = (bottleneck_time / total_op_time * 100) if total_op_time > 0 else 0

        console.print("[bold]Bottleneck Analysis:[/bold]")
        console.print(
            f"  🔍 Slowest step: [yellow]{step_names.get(bottleneck_key, bottleneck_key)}[/yellow]"
        )
        console.print(f"     Component: {components.get(bottleneck_key, 'Unknown')}")
        console.print(f"     Time: {bottleneck_time:.2f}s ({bottleneck_pct:.1f}% of operation)")

        # Suggestions based on bottleneck
        if "llm" in bottleneck_key:
            console.print(
                f"\n  💡 [dim]Suggestion: Consider using a faster LLM model or increasing API rate limits[/dim]"
            )
        elif "embedding" in bottleneck_key:
            console.print(
                f"\n  💡 [dim]Suggestion: Consider using a faster embedding model or batch processing[/dim]"
            )
        elif "vector" in bottleneck_key:
            console.print(
                f"\n  💡 [dim]Suggestion: Check vector store connection latency or index optimization[/dim]"
            )
        elif "graph" in bottleneck_key:
            console.print(
                f"\n  💡 [dim]Suggestion: Graph operations can be slow - consider if graph store is needed[/dim]"
            )

    console.print()


def version_callback(value: bool) -> None:
    """Show version and exit."""
    if value:
        console.print(f"[bold green]mem0-open-mcp[/bold green] version [cyan]{__version__}[/cyan]")
        raise typer.Exit()


@app.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    version: Annotated[
        bool,
        typer.Option(
            "--version",
            "-v",
            help="Show version and exit.",
        ),
    ] = False,
) -> None:
    """Standalone MCP server for mem0 with web configuration UI."""
    if version:
        console.print(f"[bold green]mem0-open-mcp[/bold green] version [cyan]{__version__}[/cyan]")
        raise typer.Exit()

    # Show help if no command provided
    if ctx.invoked_subcommand is None:
        console.print(ctx.get_help())
        console.print()  # Empty line

        # Version info at the bottom
        console.print(f"[dim]Current version: [cyan]{__version__}[/cyan][/dim]")

        # Check for updates
        latest, is_update_available = _check_for_updates()
        if is_update_available and latest:
            console.print(
                f"[yellow]⚠ Update available: {__version__} → {latest}[/yellow]\n"
                f"[dim]Run [cyan]mem0-open-mcp update[/cyan] to upgrade[/dim]"
            )

        raise typer.Exit()


@app.command()
def serve(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file (YAML or JSON).",
            exists=True,
        ),
    ] = None,
    host: Annotated[
        str | None,
        typer.Option("--host", "-h", help="Host to bind to."),
    ] = None,
    port: Annotated[
        int | None,
        typer.Option("--port", "-p", help="Port to listen on."),
    ] = None,
    user_id: Annotated[
        str | None,
        typer.Option("--user-id", "-u", help="Default user ID for memories."),
    ] = None,
    reload: Annotated[
        bool,
        typer.Option("--reload", "-r", help="Enable auto-reload for development."),
    ] = False,
    log_level: Annotated[
        str,
        typer.Option("--log-level", "-l", help="Logging level."),
    ] = "info",
) -> None:
    """Start the MCP server.

    Examples:
        mem0-open-mcp serve
        mem0-open-mcp serve --port 8765 --user-id alice
        mem0-open-mcp serve --config ./my-config.yaml
    """
    # Load configuration
    loader = ConfigLoader(config_file)
    config = loader.load()

    # Override with CLI options
    if host:
        config.server.host = host
    if port:
        config.server.port = port
    if user_id:
        config.server.user_id = user_id
    if reload:
        config.server.reload = reload
    if log_level:
        config.server.log_level = log_level  # type: ignore

    logging.basicConfig(
        level=getattr(logging, config.server.log_level.upper()),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    )
    log_file = _setup_file_logging(config.server.log_level, mode="server")

    # Display startup info
    console.print(
        Panel.fit(
            f"[bold green]mem0-open-mcp[/bold green] v{__version__}\n"
            f"[dim]MCP server for mem0 memory management[/dim]",
            border_style="green",
        )
    )

    _check_for_updates()  # Check for updates (returns tuple but we don't use it here)

    console.print("[bold]Configuration:[/bold]")
    console.print(f"  Host: [cyan]{config.server.host}[/cyan]")
    console.print(f"  Port: [cyan]{config.server.port}[/cyan]")
    console.print(f"  User ID: [cyan]{config.server.user_id}[/cyan]")
    console.print(f"  LLM: [cyan]{config.llm.provider.value}[/cyan] / {config.llm.config.model}")
    console.print(
        f"  Embedder: [cyan]{config.embedder.provider.value}[/cyan] / {config.embedder.config.model}"
    )
    console.print(f"  Vector Store: [cyan]{config.vector_store.provider.value}[/cyan]")
    console.print()

    # Start the server
    try:
        from mem0_server.server import run_server

        run_server(config, loader)
    except ImportError as e:
        console.print(f"[red]Error starting server: {e}[/red]")
        console.print(
            "[yellow]Make sure all dependencies are installed: pip install mem0-open-mcp[/yellow]"
        )
        raise typer.Exit(1) from None
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@app.command()
def test(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file (YAML or JSON).",
            exists=True,
        ),
    ] = None,
    use_async: Annotated[
        bool,
        typer.Option(
            "--async",
            "-a",
            help="Use AsyncMemory instead of sync Memory.",
        ),
    ] = False,
    message: Annotated[
        str | None,
        typer.Option(
            "--message",
            "-m",
            help="Custom test message for memory operations.",
        ),
    ] = None,
    profile: Annotated[
        bool,
        typer.Option(
            "--profile",
            "-p",
            help="Run detailed profiling of each component (LLM, Embedder, VectorStore).",
        ),
    ] = False,
) -> None:
    """Test connectivity and memory operations without starting server.

    Examples:
        mem0-open-mcp test
        mem0-open-mcp test --async
        mem0-open-mcp test --config ./my-config.yaml
        mem0-open-mcp test --message "John works at Google as a senior engineer."
        mem0-open-mcp test --profile  # Detailed component profiling
    """
    loader = ConfigLoader(config_file)
    config = loader.load()

    mode = "Async" if use_async else "Sync"
    if profile:
        mode = "Profile"
    console.print(
        Panel.fit(
            f"[bold green]mem0-open-mcp[/bold green] v{__version__}\n"
            f"[dim]Configuration Test ({mode})[/dim]",
            border_style="green",
        )
    )

    console.print("\n[bold]Configuration:[/bold]")
    console.print(f"  LLM: [cyan]{config.llm.provider.value}[/cyan] / {config.llm.config.model}")
    console.print(
        f"  Embedder: [cyan]{config.embedder.provider.value}[/cyan] / {config.embedder.config.model}"
    )
    console.print(f"  Vector Store: [cyan]{config.vector_store.provider.value}[/cyan]")
    if config.graph_store:
        console.print(f"  Graph Store: [cyan]{config.graph_store.provider.value}[/cyan]")
        if config.graph_store.llm:
            graph_llm_model = (
                config.graph_store.llm.config.model if config.graph_store.llm.config else "N/A"
            )
            console.print(
                f"    └─ Graph LLM: [cyan]{config.graph_store.llm.provider.value}[/cyan] / {graph_llm_model} [dim](separate)[/dim]"
            )
        else:
            console.print(f"    └─ Graph LLM: [dim](using main LLM)[/dim]")
    console.print()

    if not _run_connectivity_tests(config):
        raise typer.Exit(1)

    if profile:
        # Run detailed profiling mode
        if not _run_profiling_tests(config, message):
            raise typer.Exit(1)
    elif use_async:
        if not _run_memory_tests_async(config, message):
            raise typer.Exit(1)
    else:
        if not _run_memory_tests(config, message):
            raise typer.Exit(1)

    console.print("[bold green]All tests passed! Configuration is ready.[/bold green]")


@app.command(name="configure")
def configure(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file to create/edit.",
        ),
    ] = None,
    output: Annotated[
        Path | None,
        typer.Option(
            "--output",
            "-o",
            help="Output path for configuration file.",
        ),
    ] = None,
) -> None:
    """Interactive configuration wizard.

    Creates or edits a mem0-open-mcp configuration file with guided prompts.

    Examples:
        mem0-open-mcp configure
        mem0-open-mcp configure --output ./my-config.yaml
        mem0-open-mcp configure --config ./existing-config.yaml
    """
    console.print(
        Panel.fit(
            "[bold green]mem0-open-mcp Configuration Wizard[/bold green]\n"
            "[dim]Configure your mem0 MCP server settings[/dim]",
            border_style="green",
        )
    )

    # Load existing config if provided
    loader = ConfigLoader(config_file)
    try:
        config = loader.load()
        console.print(f"\n[dim]Loaded existing config from: {loader.config_path}[/dim]\n")
    except Exception:
        config = Mem0ServerConfig()
        console.print("\n[dim]Creating new configuration...[/dim]\n")

    # === Server Settings ===
    console.print("[bold cyan]Server Settings[/bold cyan]")

    config.server.host = Prompt.ask(
        "  Host",
        default=config.server.host,
    )

    port_str = Prompt.ask(
        "  Port",
        default=str(config.server.port),
    )
    config.server.port = int(port_str)

    config.server.user_id = Prompt.ask(
        "  Default User ID",
        default=config.server.user_id,
    )

    # === LLM Settings ===
    console.print("\n[bold cyan]LLM Settings[/bold cyan]")

    llm_providers = [p.value for p in LLMProviderType]
    console.print(f"  Available providers: {', '.join(llm_providers)}")

    llm_provider = Prompt.ask(
        "  Provider",
        default=config.llm.provider.value,
        choices=llm_providers,
    )
    config.llm.provider = LLMProviderType(llm_provider)

    # Suggest default models based on provider
    default_models = {
        "openai": "gpt-4o-mini",
        "anthropic": "claude-3-5-sonnet-20241022",
        "ollama": "llama3.2",
        "groq": "llama-3.1-70b-versatile",
    }
    suggested_model = default_models.get(llm_provider, config.llm.config.model)

    config.llm.config.model = Prompt.ask(
        "  Model",
        default=suggested_model,
    )

    if llm_provider != "ollama":
        api_key = Prompt.ask(
            "  API Key (or env:VAR_NAME)",
            default=config.llm.config.api_key or f"env:{llm_provider.upper()}_API_KEY",
        )
        config.llm.config.api_key = api_key
    else:
        base_url = Prompt.ask(
            "  Ollama URL",
            default=config.llm.config.base_url or "http://localhost:11434",
        )
        config.llm.config.base_url = base_url

    # === Embedder Settings ===
    console.print("\n[bold cyan]Embedder Settings[/bold cyan]")

    embedder_providers = [p.value for p in EmbedderProviderType]
    console.print(f"  Available providers: {', '.join(embedder_providers)}")

    embedder_provider = Prompt.ask(
        "  Provider",
        default=config.embedder.provider.value,
        choices=embedder_providers,
    )
    config.embedder.provider = EmbedderProviderType(embedder_provider)

    # Suggest default embedding models
    default_embedding_models = {
        "openai": "text-embedding-3-small",
        "ollama": "nomic-embed-text",
    }
    suggested_embed_model = default_embedding_models.get(
        embedder_provider, config.embedder.config.model
    )

    config.embedder.config.model = Prompt.ask(
        "  Model",
        default=suggested_embed_model,
    )

    if embedder_provider != "ollama":
        # Reuse LLM API key if same provider
        if embedder_provider == llm_provider and config.llm.config.api_key:
            default_embed_key = config.llm.config.api_key
        else:
            default_embed_key = (
                config.embedder.config.api_key or f"env:{embedder_provider.upper()}_API_KEY"
            )

        api_key = Prompt.ask(
            "  API Key (or env:VAR_NAME)",
            default=default_embed_key,
        )
        config.embedder.config.api_key = api_key
    else:
        base_url = Prompt.ask(
            "  Ollama URL",
            default=config.embedder.config.base_url or "http://localhost:11434",
        )
        config.embedder.config.base_url = base_url

    # === Vector Store Settings ===
    console.print("\n[bold cyan]Vector Store Settings[/bold cyan]")

    vs_providers = [p.value for p in VectorStoreProviderType]
    console.print(f"  Available providers: {', '.join(vs_providers)}")

    vs_provider = Prompt.ask(
        "  Provider",
        default=config.vector_store.provider.value,
        choices=vs_providers,
    )
    config.vector_store.provider = VectorStoreProviderType(vs_provider)

    config.vector_store.config.collection_name = Prompt.ask(
        "  Collection Name",
        default=config.vector_store.config.collection_name,
    )

    # Provider-specific settings
    if vs_provider in ("qdrant", "milvus", "chroma"):
        host = Prompt.ask(
            "  Host",
            default=config.vector_store.config.host or "localhost",
        )
        config.vector_store.config.host = host

        default_ports = {"qdrant": 6333, "milvus": 19530, "chroma": 8000}
        port_str = Prompt.ask(
            "  Port",
            default=str(config.vector_store.config.port or default_ports.get(vs_provider, 6333)),
        )
        config.vector_store.config.port = int(port_str)

    elif vs_provider == "pinecone":
        api_key = Prompt.ask(
            "  Pinecone API Key (or env:VAR_NAME)",
            default=config.vector_store.config.api_key or "env:PINECONE_API_KEY",
        )
        config.vector_store.config.api_key = api_key

    # === Custom Instructions ===
    console.print("\n[bold cyan]OpenMemory Settings[/bold cyan]")

    if Confirm.ask("  Add custom instructions for memory extraction?", default=False):
        instructions = Prompt.ask(
            "  Custom Instructions",
            default=config.openmemory.custom_instructions or "",
        )
        config.openmemory.custom_instructions = instructions if instructions else None

    # === Save Configuration ===
    console.print()

    save_path = output or config_file or Path("mem0-open-mcp.yaml")

    if Confirm.ask(f"Save configuration to [cyan]{save_path}[/cyan]?", default=True):
        saved_path = loader.save(config, save_path)
        console.print(f"\n[green]✓[/green] Configuration saved to [cyan]{saved_path}[/cyan]")
        console.print("\n[dim]Start the server with:[/dim]")
        console.print(f"  [bold]mem0-open-mcp serve --config {saved_path}[/bold]")
    else:
        console.print("[yellow]Configuration not saved.[/yellow]")


@app.command(name="config")
def config(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file to create/edit.",
        ),
    ] = None,
    output: Annotated[
        Path | None,
        typer.Option(
            "--output",
            "-o",
            help="Output path for configuration file.",
        ),
    ] = None,
) -> None:
    """Interactive configuration wizard (alias for configure).

    Creates or edits a mem0-open-mcp configuration file with guided prompts.

    Examples:
        mem0-open-mcp config
        mem0-open-mcp config --output ./my-config.yaml
        mem0-open-mcp config --config ./existing-config.yaml
    """
    return configure(config_file, output)


@app.command()
def status(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file.",
            exists=True,
        ),
    ] = None,
) -> None:
    """Show current configuration and status.

    Examples:
        mem0-open-mcp status
        mem0-open-mcp status --config ./my-config.yaml
    """
    # Load configuration
    loader = ConfigLoader(config_file)
    config = loader.load()

    console.print(
        Panel.fit(
            "[bold green]mem0-open-mcp Status[/bold green]",
            border_style="green",
        )
    )

    if loader.config_path:
        console.print(f"\n[dim]Configuration file: {loader.config_path}[/dim]")
    else:
        console.print("\n[dim]No configuration file found, using defaults[/dim]")

    # Server config table
    table = Table(title="Server Configuration")
    table.add_column("Setting", style="cyan")
    table.add_column("Value", style="green")

    table.add_row("Host", config.server.host)
    table.add_row("Port", str(config.server.port))
    table.add_row("User ID", config.server.user_id)
    table.add_row("Log Level", config.server.log_level)

    console.print(table)

    # LLM config
    llm_table = Table(title="LLM Configuration")
    llm_table.add_column("Setting", style="cyan")
    llm_table.add_column("Value", style="green")

    llm_table.add_row("Provider", config.llm.provider.value)
    llm_table.add_row("Model", config.llm.config.model)
    llm_table.add_row("Temperature", str(config.llm.config.temperature))
    llm_table.add_row("Max Tokens", str(config.llm.config.max_tokens))
    llm_table.add_row("API Key", "***" if config.llm.config.api_key else "[red]Not set[/red]")
    if config.llm.config.base_url:
        llm_table.add_row("Base URL", config.llm.config.base_url)

    console.print(llm_table)

    # Embedder config
    embed_table = Table(title="Embedder Configuration")
    embed_table.add_column("Setting", style="cyan")
    embed_table.add_column("Value", style="green")

    embed_table.add_row("Provider", config.embedder.provider.value)
    embed_table.add_row("Model", config.embedder.config.model)
    embed_table.add_row(
        "API Key", "***" if config.embedder.config.api_key else "[red]Not set[/red]"
    )
    if config.embedder.config.base_url:
        embed_table.add_row("Base URL", config.embedder.config.base_url)

    console.print(embed_table)

    # Vector store config
    vs_table = Table(title="Vector Store Configuration")
    vs_table.add_column("Setting", style="cyan")
    vs_table.add_column("Value", style="green")

    vs_table.add_row("Provider", config.vector_store.provider.value)
    vs_table.add_row("Collection", config.vector_store.config.collection_name)
    if config.vector_store.config.host:
        vs_table.add_row("Host", config.vector_store.config.host)
    if config.vector_store.config.port:
        vs_table.add_row("Port", str(config.vector_store.config.port))
    if config.vector_store.config.url:
        vs_table.add_row("URL", config.vector_store.config.url)

    console.print(vs_table)

    # Connection test hint
    console.print("\n[dim]Run 'mem0-open-mcp serve' to start the server[/dim]")


@app.command()
def update(
    check_only: Annotated[
        bool,
        typer.Option(
            "--check",
            "-c",
            help="Only check for updates without installing.",
        ),
    ] = False,
    force: Annotated[
        bool,
        typer.Option(
            "--force",
            "-f",
            help="Force reinstall even if already up to date.",
        ),
    ] = False,
) -> None:
    """Check for updates and upgrade to the latest version.

    Examples:
        mem0-open-mcp update
        mem0-open-mcp update --check
        mem0-open-mcp update --force
    """
    import subprocess
    import sys

    console.print(
        Panel.fit(
            "[bold green]mem0-open-mcp Update[/bold green]",
            border_style="green",
        )
    )

    console.print(f"\n  Current version: [cyan]{__version__}[/cyan]")

    # Check PyPI for latest version
    console.print("  Checking PyPI for latest version...", end=" ")
    try:
        import httpx

        resp = httpx.get("https://pypi.org/pypi/mem0-open-mcp/json", timeout=10)
        resp.raise_for_status()
        latest = resp.json()["info"]["version"]
        console.print(f"[green]✓[/green]")
        console.print(f"  Latest version:  [cyan]{latest}[/cyan]")
    except Exception as e:
        console.print(f"[red]✗[/red]")
        console.print(f"  [red]Failed to check PyPI: {e}[/red]")
        raise typer.Exit(1)

    # Compare versions
    current_parsed = _parse_version(__version__)
    latest_parsed = _parse_version(latest)

    console.print()

    if current_parsed >= latest_parsed and not force:
        console.print("[green]✓ You are already using the latest version![/green]")

        # Update cache
        try:
            UPDATE_CHECK_CACHE.parent.mkdir(parents=True, exist_ok=True)
            UPDATE_CHECK_CACHE.write_text(
                json.dumps(
                    {
                        "last_check": time.time(),
                        "latest": latest,
                    }
                )
            )
        except Exception:
            pass

        return

    if check_only:
        if current_parsed < latest_parsed:
            console.print(f"[yellow]Update available: {__version__} → {latest}[/yellow]")
            console.print("\n[dim]Run 'mem0-open-mcp update' to install the update.[/dim]")
        return

    # Perform upgrade
    action = "Reinstalling" if force else "Upgrading"
    console.print(f"[bold]{action} mem0-open-mcp...[/bold]\n")

    try:
        import shutil

        # Detect package manager: prefer uv if available
        uv_path = shutil.which("uv")

        if uv_path:
            # Use uv
            console.print("  [dim]Using uv...[/dim]")
            cmd = [uv_path, "pip", "install", "--upgrade", "mem0-open-mcp"]
            if force:
                cmd.append("--force-reinstall")
            pkg_manager = "uv"
        else:
            # Fallback to pip
            console.print("  [dim]Using pip...[/dim]")
            cmd = [sys.executable, "-m", "pip", "install", "--upgrade", "mem0-open-mcp"]
            if force:
                cmd.append("--force-reinstall")
            pkg_manager = "pip"

        result = subprocess.run(
            cmd,
            capture_output=True,
            text=True,
        )

        if result.returncode == 0:
            console.print(f"[green]✓ Successfully upgraded to {latest}![/green]")
            console.print("\n[dim]Restart the server to use the new version.[/dim]")

            # Update cache
            try:
                UPDATE_CHECK_CACHE.parent.mkdir(parents=True, exist_ok=True)
                UPDATE_CHECK_CACHE.write_text(
                    json.dumps(
                        {
                            "last_check": time.time(),
                            "latest": latest,
                        }
                    )
                )
            except Exception:
                pass
        else:
            console.print(f"[red]✗ Upgrade failed[/red]")
            if result.stderr:
                console.print(f"[dim]{result.stderr}[/dim]")
            # Suggest alternative
            alt_cmd = (
                "pip install --upgrade mem0-open-mcp"
                if pkg_manager == "uv"
                else "uv pip install --upgrade mem0-open-mcp"
            )
            console.print(f"\n[dim]Try: {alt_cmd}[/dim]")
            raise typer.Exit(1)

    except Exception as e:
        console.print(f"[red]✗ Error during upgrade: {e}[/red]")
        console.print("\n[dim]Try manually:[/dim]")
        console.print("[dim]  pip install --upgrade mem0-open-mcp[/dim]")
        console.print("[dim]  uv pip install --upgrade mem0-open-mcp[/dim]")
        raise typer.Exit(1)


@app.command()
def stdio(
    config_file: Annotated[
        Path | None,
        typer.Option(
            "--config",
            "-c",
            help="Path to configuration file (YAML or JSON).",
            exists=True,
        ),
    ] = None,
    user_id: Annotated[
        str | None,
        typer.Option("--user-id", "-u", help="Default user ID for memories."),
    ] = None,
    log_level: Annotated[
        str,
        typer.Option("--log-level", "-l", help="Logging level."),
    ] = "error",
) -> None:
    """Run MCP server in stdio mode (for mcp-proxy or Claude Desktop).

    This mode allows the server to be spawned as a subprocess and communicate
    via stdin/stdout using JSON-RPC messages.

    Examples:
        mem0-open-mcp stdio
        mem0-open-mcp stdio --config ./my-config.yaml
        mem0-open-mcp stdio --user-id alice
    """
    import asyncio
    import sys

    # Load configuration
    loader = ConfigLoader(config_file)
    config = loader.load()

    # Override with CLI options
    if user_id:
        config.server.user_id = user_id

    logging.basicConfig(
        level=getattr(logging, log_level.upper()),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        stream=sys.stderr,
    )
    _setup_file_logging(log_level, mode="stdio")

    async def run_stdio():
        """Run the MCP server in stdio mode."""
        try:
            from mcp.server.stdio import stdio_server
            from mem0_server.server import create_mcp_manager_stdio

            # Create MCP manager optimized for stdio transport
            manager = create_mcp_manager_stdio(config)

            # Run stdio server
            async with stdio_server() as (read_stream, write_stream):
                await manager.mcp._mcp_server.run(
                    read_stream,
                    write_stream,
                    manager.mcp._mcp_server.create_initialization_options(),
                )
        except Exception as e:
            logger.error(f"Error in stdio mode: {e}")
            raise typer.Exit(1) from None

    # Run the async function
    try:
        asyncio.run(run_stdio())
    except KeyboardInterrupt:
        pass
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        raise typer.Exit(1) from None


@app.command()
def init(
    path: Annotated[
        Path | None,
        typer.Argument(
            help="Path to create configuration file. Defaults to ~/.mem0-open-mcp/mem0-open-mcp.yaml",
        ),
    ] = None,
    force: Annotated[
        bool,
        typer.Option("--force", "-f", help="Overwrite existing file."),
    ] = False,
) -> None:
    """Create a default configuration file.

    Examples:
        mem0-open-mcp init
        mem0-open-mcp init ./config.yaml
        mem0-open-mcp init --force
    """
    from mem0_server.config.loader import MEM0_HOME

    default_path = MEM0_HOME / "mem0-open-mcp.yaml"
    target_path = path if path else default_path

    if target_path.exists() and not force:
        console.print(f"[red]File already exists: {target_path}[/red]")
        console.print("[dim]Use --force to overwrite[/dim]")
        raise typer.Exit(1)

    saved_path = ConfigLoader.create_default_config_file(target_path)
    console.print(f"[green]✓[/green] Created configuration file:")
    console.print(f"  [cyan]{saved_path.absolute()}[/cyan]")
    console.print("\n[dim]Edit the file to customize your settings, then run:[/dim]")
    console.print("  [bold]mem0-open-mcp serve[/bold]")


if __name__ == "__main__":
    app()



"""Tests for mem0-server configuration."""

import os
import tempfile
from pathlib import Path

import pytest
import yaml

from mem0_server.config import (
    ConfigLoader,
    Mem0ServerConfig,
    LLMProvider,
    LLMConfig,
    LLMProviderType,
    EmbedderProvider,
    EmbedderConfig,
    EmbedderProviderType,
    VectorStoreProvider,
    VectorStoreConfig,
    VectorStoreProviderType,
)
from mem0_server.config.schema import get_default_config


class TestConfigSchema:
    """Tests for configuration schema."""
    
    def test_default_config(self):
        """Test default configuration creation."""
        config = get_default_config()
        
        assert config.server.host == "0.0.0.0"
        assert config.server.port == 8765
        assert config.server.user_id == "default"
        
        assert config.llm.provider == LLMProviderType.OPENAI
        assert config.llm.config.model == "gpt-4o-mini"
        
        assert config.embedder.provider == EmbedderProviderType.OPENAI
        assert config.embedder.config.model == "text-embedding-3-small"
        
        assert config.vector_store.provider == VectorStoreProviderType.QDRANT
    
    def test_llm_config_validation(self):
        """Test LLM configuration validation."""
        config = LLMConfig(
            model="gpt-4o",
            temperature=0.5,
            max_tokens=4000,
        )
        
        assert config.model == "gpt-4o"
        assert config.temperature == 0.5
        assert config.max_tokens == 4000
    
    def test_temperature_bounds(self):
        """Test temperature validation bounds."""
        # Valid temperatures
        LLMConfig(model="test", temperature=0.0)
        LLMConfig(model="test", temperature=1.0)
        LLMConfig(model="test", temperature=2.0)
        
        # Invalid temperature
        with pytest.raises(ValueError):
            LLMConfig(model="test", temperature=2.5)
    
    def test_to_mem0_config(self):
        """Test conversion to mem0 library config format."""
        config = get_default_config()
        mem0_config = config.to_mem0_config()
        
        assert "llm" in mem0_config
        assert "embedder" in mem0_config
        assert "vector_store" in mem0_config
        assert mem0_config["version"] == "v1.1"
        
        assert mem0_config["llm"]["provider"] == "openai"
        assert mem0_config["embedder"]["provider"] == "openai"
        assert mem0_config["vector_store"]["provider"] == "qdrant"


class TestConfigLoader:
    """Tests for configuration loader."""
    
    def test_load_default_config(self):
        """Test loading default config when no file exists."""
        loader = ConfigLoader()
        config = loader.load()
        
        assert isinstance(config, Mem0ServerConfig)
        assert config.server.port == 8765
    
    def test_save_and_load_yaml(self):
        """Test saving and loading YAML config."""
        with tempfile.TemporaryDirectory() as tmpdir:
            config_path = Path(tmpdir) / "test-config.yaml"
            
            # Create and save config
            config = get_default_config()
            config.server.port = 9999
            config.server.user_id = "test-user"
            
            loader = ConfigLoader()
            saved_path = loader.save(config, config_path)
            
            assert saved_path == config_path
            assert config_path.exists()
            
            # Load and verify
            new_loader = ConfigLoader(config_path)
            loaded_config = new_loader.load()
            
            assert loaded_config.server.port == 9999
            assert loaded_config.server.user_id == "test-user"
    
    def test_env_var_resolution(self):
        """Test environment variable resolution."""
        with tempfile.TemporaryDirectory() as tmpdir:
            config_path = Path(tmpdir) / "test-config.yaml"
            
            # Set env var
            os.environ["TEST_API_KEY"] = "test-key-12345"
            
            # Write config with env var reference
            config_data = {
                "llm": {
                    "provider": "openai",
                    "config": {
                        "model": "gpt-4o",
                        "api_key": "${TEST_API_KEY}",
                    }
                }
            }
            
            with open(config_path, "w") as f:
                yaml.dump(config_data, f)
            
            # Load and verify
            loader = ConfigLoader(config_path)
            config = loader.load()
            
            # env vars in ${} syntax are resolved during load
            # Note: api_key validation happens at model level with env: syntax
            
            # Cleanup
            del os.environ["TEST_API_KEY"]
    
    def test_create_default_config_file(self):
        """Test creating default config file."""
        with tempfile.TemporaryDirectory() as tmpdir:
            config_path = Path(tmpdir) / "new-config.yaml"
            
            saved_path = ConfigLoader.create_default_config_file(config_path)
            
            assert saved_path == config_path
            assert config_path.exists()
            
            # Verify content
            with open(config_path) as f:
                data = yaml.safe_load(f)
            
            assert "server" in data
            assert "llm" in data
            assert "embedder" in data
            assert "vector_store" in data


class TestProviderTypes:
    """Tests for provider type enums."""
    
    def test_llm_providers(self):
        """Test LLM provider types."""
        providers = list(LLMProviderType)
        
        assert LLMProviderType.OPENAI in providers
        assert LLMProviderType.ANTHROPIC in providers
        assert LLMProviderType.OLLAMA in providers
        assert len(providers) >= 10
    
    def test_embedder_providers(self):
        """Test embedder provider types."""
        providers = list(EmbedderProviderType)
        
        assert EmbedderProviderType.OPENAI in providers
        assert EmbedderProviderType.OLLAMA in providers
        assert len(providers) >= 8
    
    def test_vector_store_providers(self):
        """Test vector store provider types."""
        providers = list(VectorStoreProviderType)
        
        assert VectorStoreProviderType.QDRANT in providers
        assert VectorStoreProviderType.CHROMA in providers
        assert VectorStoreProviderType.PINECONE in providers
        assert len(providers) >= 10



"""Shared memory operations used by both CLI tests and MCP server."""

from __future__ import annotations

import asyncio
import json
import logging
import time
from copy import deepcopy
from datetime import datetime
from typing import Any

import pytz

from mem0.configs.prompts import get_update_memory_messages
from mem0.memory.main import _build_filters_and_metadata
from mem0.memory.utils import (
    extract_json,
    get_fact_retrieval_messages,
    parse_messages,
    parse_vision_messages,
    remove_code_blocks,
)

from mem0_server.config import Mem0ServerConfig

logger = logging.getLogger(__name__)
perf_logger = logging.getLogger("mem0_server.performance")


async def add_memory_with_profiling(
    *,
    memory_client: Any,
    text: Any,
    user_id: str,
    metadata: dict[str, Any] | None,
    config: Mem0ServerConfig,
) -> dict[str, Any]:
    """Add memory with detailed profiling steps.

    This is the shared implementation used by both MCP tools and CLI tests.
    """
    _ = config
    total_start = time.perf_counter()

    def _log_step(step: str, step_start: float, **extras: Any) -> None:
        elapsed_ms = (time.perf_counter() - step_start) * 1000
        log_parts = [
            f"user={user_id}",
            f"step={step}",
            f"time={elapsed_ms:.1f}ms",
        ]
        for key, value in extras.items():
            log_parts.append(f"{key}={value}")
        perf_logger.info("[add] " + " | ".join(log_parts))

    def _log_request(input_text: Any) -> None:
        if isinstance(input_text, str):
            clean = input_text.replace("\n", " ").replace("\t", " ")
            preview = clean[:80] + "..." if len(clean) > 80 else clean
        elif isinstance(input_text, list):
            preview = f"[{len(input_text)} messages]"
        else:
            preview = f"[{type(input_text).__name__}]"
        perf_logger.info(f"[add] user={user_id} | request: {preview}")

    def _log_facts(facts: list[str]) -> None:
        if not facts:
            return
        truncated = [f[:50] + "..." if len(f) > 50 else f for f in facts[:5]]
        perf_logger.info(f"[add] user={user_id} | facts_preview={truncated}")

    def _log_actions(memories: list[dict]) -> None:
        if not memories:
            return
        summary = {"ADD": 0, "UPDATE": 0, "DELETE": 0}
        for m in memories:
            event = m.get("event", "")
            if event in summary:
                summary[event] += 1
        parts = [f"{k}={v}" for k, v in summary.items() if v > 0]
        if parts:
            perf_logger.info(f"[add] user={user_id} | actions: {', '.join(parts)}")
        for m in memories[:3]:
            text = m.get("memory", "")[:60]
            event = m.get("event", "")
            perf_logger.info(f"[add] user={user_id} |   {event}: {text}...")

    processed_metadata, effective_filters = _build_filters_and_metadata(
        user_id=user_id,
        input_metadata=metadata or {},
    )

    if isinstance(text, str):
        messages = [{"role": "user", "content": text}]
    elif isinstance(text, dict):
        messages = [text]
    else:
        messages = text

    if memory_client.config.llm.config.get("enable_vision"):
        messages = parse_vision_messages(
            messages, memory_client.llm, memory_client.config.llm.config.get("vision_details")
        )
    else:
        messages = parse_vision_messages(messages)

    _log_request(text)

    fact_start = time.perf_counter()
    parsed_messages = parse_messages(messages)
    if memory_client.config.custom_fact_extraction_prompt:
        system_prompt = memory_client.config.custom_fact_extraction_prompt
        user_prompt = f"Input:\n{parsed_messages}"
    else:
        should_use_agent = getattr(memory_client, "_should_use_agent_memory_extraction", None)
        is_agent_memory = False
        if callable(should_use_agent):
            is_agent_memory = should_use_agent(messages, processed_metadata)
        system_prompt, user_prompt = get_fact_retrieval_messages(parsed_messages, is_agent_memory)

    response = await asyncio.to_thread(
        memory_client.llm.generate_response,
        messages=[
            {"role": "system", "content": system_prompt},
            {"role": "user", "content": user_prompt},
        ],
        response_format={"type": "json_object"},
    )

    try:
        response = remove_code_blocks(response)
        if not response.strip():
            new_retrieved_facts = []
        else:
            try:
                new_retrieved_facts = json.loads(response)["facts"]
            except json.JSONDecodeError:
                extracted_json = extract_json(response)
                new_retrieved_facts = json.loads(extracted_json)["facts"]
    except Exception as e:
        logger.error(f"Error in new_retrieved_facts (profiling): {e}")
        new_retrieved_facts = []

    _log_step("fact_extraction", fact_start, facts=len(new_retrieved_facts))
    _log_facts(new_retrieved_facts)

    embed_start = time.perf_counter()
    new_message_embeddings: dict[str, Any] = {}
    for fact in new_retrieved_facts:
        embeddings = await asyncio.to_thread(memory_client.embedding_model.embed, fact, "add")
        new_message_embeddings[fact] = embeddings
    _log_step("embedding", embed_start, count=len(new_message_embeddings))

    search_start = time.perf_counter()
    retrieved_old_memory = []
    search_filters = {}
    if effective_filters.get("user_id"):
        search_filters["user_id"] = effective_filters["user_id"]
    if effective_filters.get("agent_id"):
        search_filters["agent_id"] = effective_filters["agent_id"]
    if effective_filters.get("run_id"):
        search_filters["run_id"] = effective_filters["run_id"]

    total_found = 0
    for fact in new_retrieved_facts:
        embeddings = new_message_embeddings.get(fact)
        existing_mems = await asyncio.to_thread(
            memory_client.vector_store.search,
            query=fact,
            vectors=embeddings,
            limit=5,
            filters=search_filters,
        )
        total_found += len(existing_mems)
        retrieved_old_memory.extend(
            [{"id": mem.id, "text": mem.payload.get("data", "")} for mem in existing_mems]
        )

    unique_data = {item["id"]: item for item in retrieved_old_memory}
    retrieved_old_memory = list(unique_data.values())
    temp_uuid_mapping = {}
    for idx, item in enumerate(retrieved_old_memory):
        temp_uuid_mapping[str(idx)] = item["id"]
        retrieved_old_memory[idx]["id"] = str(idx)

    _log_step("vector_search", search_start, matches=total_found)

    update_start = time.perf_counter()
    if new_retrieved_facts:
        function_calling_prompt = get_update_memory_messages(
            retrieved_old_memory,
            new_retrieved_facts,
            memory_client.config.custom_update_memory_prompt,
        )
        try:
            update_response = await asyncio.to_thread(
                memory_client.llm.generate_response,
                messages=[{"role": "user", "content": function_calling_prompt}],
                response_format={"type": "json_object"},
            )
        except Exception as e:
            logger.error(f"Error in new memory actions response (profiling): {e}")
            update_response = ""
        try:
            if not update_response or not update_response.strip():
                new_memories_with_actions = {}
            else:
                update_response = remove_code_blocks(update_response)
                new_memories_with_actions = json.loads(update_response)
        except Exception as e:
            logger.error(f"Invalid JSON response (profiling): {e}")
            new_memories_with_actions = {}
    else:
        new_memories_with_actions = {}

    _log_step("update_decision", update_start)

    insert_start = time.perf_counter()
    returned_memories = []
    memory_tasks = []

    for resp in new_memories_with_actions.get("memory", []):
        try:
            action_text = resp.get("text")
            if not action_text:
                continue
            event_type = resp.get("event")

            if event_type == "ADD":
                task = asyncio.create_task(
                    memory_client._create_memory(
                        data=action_text,
                        existing_embeddings=new_message_embeddings,
                        metadata=deepcopy(processed_metadata),
                    )
                )
                memory_tasks.append((task, resp, "ADD", None))
            elif event_type == "UPDATE":
                task = asyncio.create_task(
                    memory_client._update_memory(
                        memory_id=temp_uuid_mapping[resp["id"]],
                        data=action_text,
                        existing_embeddings=new_message_embeddings,
                        metadata=deepcopy(processed_metadata),
                    )
                )
                memory_tasks.append((task, resp, "UPDATE", temp_uuid_mapping[resp["id"]]))
            elif event_type == "DELETE":
                memory_id = temp_uuid_mapping.get(resp.get("id"))
                if memory_id:
                    task = asyncio.create_task(memory_client._delete_memory(memory_id=memory_id))
                    memory_tasks.append((task, resp, "DELETE", memory_id))
            elif event_type == "NONE":
                memory_id = temp_uuid_mapping.get(resp.get("id"))
                if memory_id and (
                    processed_metadata.get("agent_id") or processed_metadata.get("run_id")
                ):

                    async def update_session_ids(mem_id, meta):
                        existing_memory = await asyncio.to_thread(
                            memory_client.vector_store.get, vector_id=mem_id
                        )
                        updated_metadata = deepcopy(existing_memory.payload)
                        if meta.get("agent_id"):
                            updated_metadata["agent_id"] = meta["agent_id"]
                        if meta.get("run_id"):
                            updated_metadata["run_id"] = meta["run_id"]
                        updated_metadata["updated_at"] = datetime.now(
                            pytz.timezone("US/Pacific")
                        ).isoformat()

                        await asyncio.to_thread(
                            memory_client.vector_store.update,
                            vector_id=mem_id,
                            vector=None,
                            payload=updated_metadata,
                        )

                    task = asyncio.create_task(update_session_ids(memory_id, processed_metadata))
                    memory_tasks.append((task, resp, "NONE", memory_id))
        except Exception as e:
            logger.error(f"Error processing memory action (profiling): {resp}, Error: {e}")

    for task, resp, event_type, mem_id in memory_tasks:
        try:
            result_id = await task
            if event_type == "ADD":
                returned_memories.append(
                    {"id": result_id, "memory": resp.get("text"), "event": event_type}
                )
            elif event_type == "UPDATE":
                returned_memories.append(
                    {
                        "id": mem_id,
                        "memory": resp.get("text"),
                        "event": event_type,
                        "previous_memory": resp.get("old_memory"),
                    }
                )
            elif event_type == "DELETE":
                returned_memories.append(
                    {"id": mem_id, "memory": resp.get("text"), "event": event_type}
                )
        except Exception as e:
            logger.error(f"Error awaiting memory task (profiling): {e}")

    _log_step("vector_insert", insert_start, inserted=len(returned_memories))
    _log_actions(returned_memories)

    graph_start = time.perf_counter()
    graph_relations = []
    if getattr(memory_client, "enable_graph", False) and getattr(memory_client, "graph", None):
        try:
            graph_relations = await memory_client._add_to_graph(
                messages,
                deepcopy(effective_filters),
            )
        except Exception as e:
            logger.error(f"Error adding to graph (profiling): {e}")
            graph_relations = []
    _log_step("graph", graph_start, relations=len(graph_relations))

    total_ms = (time.perf_counter() - total_start) * 1000
    perf_logger.info(f"[add] user={user_id} | total={total_ms:.1f}ms | status=success")

    if memory_client.enable_graph:
        return {"results": returned_memories, "relations": graph_relations}
    return {"results": returned_memories}


async def add_memory_op(
    *,
    memory_client: Any,
    text: str,
    user_id: str,
    metadata: dict[str, Any] | None,
    config: Mem0ServerConfig,
) -> dict[str, Any]:
    """Shared add memory operation with optional profiling."""
    if config.server.performance_logging:
        try:
            return await add_memory_with_profiling(
                memory_client=memory_client,
                text=text,
                user_id=user_id,
                metadata=metadata,
                config=config,
            )
        except Exception as e:
            logger.warning(f"Performance logging failed, falling back to standard add: {e}")

    if metadata:
        return await memory_client.add(text, user_id=user_id, metadata=metadata)
    return await memory_client.add(text, user_id=user_id)


async def search_memory_op(
    *,
    memory_client: Any,
    query: str,
    user_id: str,
    limit: int,
    config: Mem0ServerConfig,
) -> dict[str, Any]:
    """Search memories with optional performance logging."""
    _ = config
    results = await memory_client.search(query=query, user_id=user_id, limit=limit)
    if not results or not results.get("results"):
        return {"results": [], "message": "No relevant memories found."}
    return results


async def list_memories_op(
    *,
    memory_client: Any,
    user_id: str,
    config: Mem0ServerConfig,
) -> dict[str, Any]:
    """List all memories for user."""
    _ = config
    return await memory_client.get_all(user_id=user_id)


async def get_memory_op(
    *,
    memory_client: Any,
    memory_id: str,
    config: Mem0ServerConfig,
) -> dict[str, Any]:
    """Get a specific memory by ID."""
    _ = config
    return await memory_client.get(memory_id)


async def delete_memory_op(
    *,
    memory_client: Any,
    memory_id: str,
    config: Mem0ServerConfig,
) -> str:
    """Delete a single memory by ID."""
    _ = config
    await memory_client.delete(memory_id)
    return memory_id


async def delete_memories_op(
    *,
    memory_client: Any,
    memory_ids: list[str],
    config: Mem0ServerConfig,
) -> list[str]:
    """Delete multiple memories by IDs."""
    _ = config
    deleted = []
    for memory_id in memory_ids:
        try:
            await memory_client.delete(memory_id)
            deleted.append(memory_id)
        except Exception as e:
            logger.warning(f"Failed to delete memory {memory_id}: {e}")
    return deleted


async def delete_all_memories_op(
    *,
    memory_client: Any,
    user_id: str,
    config: Mem0ServerConfig,
    use_individual_delete: bool = False,
) -> int:
    """Delete all memories for the user.

    Args:
        memory_client: The memory client instance.
        user_id: The user ID to delete memories for.
        config: The server configuration.
        use_individual_delete: If True, fetch all memories and delete individually.
            This is a workaround for mem0's delete_all() bug that resets the
            entire collection instead of just the user's memories.

    Returns:
        Number of memories deleted (only when use_individual_delete=True).
    """
    _ = config
    if use_individual_delete:
        # Workaround for mem0 delete_all() bug - fetch and delete individually
        all_memories = await memory_client.get_all(user_id=user_id)
        results = all_memories.get("results", []) if all_memories else []
        deleted_count = 0
        for memory in results:
            memory_id = memory.get("id")
            if memory_id:
                try:
                    await memory_client.delete(memory_id)
                    deleted_count += 1
                except Exception as e:
                    logger.warning(f"Failed to delete memory {memory_id}: {e}")
        return deleted_count
    else:
        await memory_client.delete_all(user_id=user_id)
        return -1  # Unknown count when using bulk delete



"""Tests for mem0-server API routes."""

import pytest
from fastapi.testclient import TestClient

from mem0_server.config import Mem0ServerConfig, ConfigLoader
from mem0_server.config.schema import get_default_config
from mem0_server.server import create_app


@pytest.fixture
def config():
    """Get default config for testing."""
    return get_default_config()


@pytest.fixture
def mock_memory_client():
    """Mock memory client that returns empty results."""
    class MockMemoryClient:
        def add(self, text, user_id=None, metadata=None):
            return {"results": [{"id": "test-id", "memory": text, "event": "ADD"}]}
        
        def search(self, query, user_id=None, limit=10):
            return {"results": []}
        
        def get_all(self, user_id=None):
            return {"results": []}
        
        def get(self, memory_id):
            return {"id": memory_id, "memory": "Test memory"}
        
        def delete(self, memory_id):
            pass
        
        def delete_all(self, user_id=None):
            pass
    
    return MockMemoryClient()


@pytest.fixture
def client(config, mock_memory_client):
    """Create test client with mocked memory client."""
    app = create_app(config)
    
    # Override the memory client getter
    # Note: In real tests, we'd need to properly inject the mock
    
    return TestClient(app)


class TestHealthEndpoints:
    """Tests for health and status endpoints."""
    
    def test_health_check(self, client):
        """Test health check endpoint."""
        response = client.get("/health")
        assert response.status_code == 200
        
        data = response.json()
        assert data["status"] == "ok"
        assert data["service"] == "mem0-server"
    
    def test_root_endpoint(self, client):
        """Test root endpoint returns service info."""
        response = client.get("/")
        assert response.status_code == 200
        
        data = response.json()
        assert "service" in data
        assert "version" in data
        assert "docs" in data


class TestConfigEndpoints:
    """Tests for configuration endpoints."""
    
    def test_get_config(self, client):
        """Test getting configuration."""
        response = client.get("/api/v1/config")
        assert response.status_code == 200
        
        data = response.json()
        assert "server" in data
        assert "llm" in data
        assert "embedder" in data
        assert "vector_store" in data
        assert "openmemory" in data
    
    def test_get_llm_config(self, client):
        """Test getting LLM configuration."""
        response = client.get("/api/v1/config/llm")
        assert response.status_code == 200
        
        data = response.json()
        assert "provider" in data
        assert "config" in data
    
    def test_get_embedder_config(self, client):
        """Test getting embedder configuration."""
        response = client.get("/api/v1/config/embedder")
        assert response.status_code == 200
        
        data = response.json()
        assert "provider" in data
        assert "config" in data
    
    def test_get_vector_store_config(self, client):
        """Test getting vector store configuration."""
        response = client.get("/api/v1/config/vector_store")
        assert response.status_code == 200
        
        data = response.json()
        assert "provider" in data
        assert "config" in data
    
    def test_get_providers(self, client):
        """Test getting available providers."""
        response = client.get("/api/v1/providers")
        assert response.status_code == 200
        
        data = response.json()
        assert "llm_providers" in data
        assert "embedder_providers" in data
        assert "vector_store_providers" in data
        
        assert "openai" in data["llm_providers"]
        assert "ollama" in data["llm_providers"]
    
    def test_get_status(self, client):
        """Test getting server status."""
        response = client.get("/api/v1/status")
        assert response.status_code == 200
        
        data = response.json()
        assert "status" in data
        assert "version" in data
        assert "providers" in data


class TestMemoryEndpoints:
    """Tests for memory endpoints."""
    
    def test_list_memories_service_unavailable(self, client):
        """Test listing memories when service is unavailable."""
        # Without a properly initialized memory client, should return 503
        response = client.get("/api/v1/memories")
        # May return 503 if memory client not available
        assert response.status_code in [200, 503]
    
    def test_search_memories_request(self, client):
        """Test search memories endpoint accepts proper request."""
        response = client.post(
            "/api/v1/memories/search",
            json={"query": "test query", "limit": 5}
        )
        # May return 503 if memory client not available
        assert response.status_code in [200, 503]
    
    def test_add_memory_request(self, client):
        """Test add memory endpoint accepts proper request."""
        response = client.post(
            "/api/v1/memories",
            json={"text": "Test memory content"}
        )
        # May return 503 if memory client not available
        assert response.status_code in [200, 503]


class TestOpenAPISpec:
    """Tests for OpenAPI specification."""
    
    def test_openapi_spec_available(self, client):
        """Test that OpenAPI spec is available."""
        response = client.get("/openapi.json")
        assert response.status_code == 200
        
        data = response.json()
        assert "openapi" in data
        assert "paths" in data
        assert "info" in data
    
    def test_docs_available(self, client):
        """Test that docs are available."""
        response = client.get("/docs")
        assert response.status_code == 200



"""Configuration schema for mem0-server using Pydantic models."""

from __future__ import annotations

import os
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Field, field_validator

# =============================================================================
# LLM Providers
# =============================================================================


class LLMProviderType(str, Enum):
    """Supported LLM providers."""

    OPENAI = "openai"
    ANTHROPIC = "anthropic"
    AZURE_OPENAI = "azure_openai"
    OLLAMA = "ollama"
    TOGETHER = "together"
    GROQ = "groq"
    LITELLM = "litellm"
    MISTRALAI = "mistralai"
    GOOGLE_AI = "google_ai"
    AWS_BEDROCK = "aws_bedrock"
    GEMINI = "gemini"
    DEEPSEEK = "deepseek"
    XAI = "xai"
    LMSTUDIO = "lmstudio"


class LLMConfig(BaseModel):
    """LLM configuration settings."""

    model: str = Field(default="gpt-4o-mini", description="Model name")
    temperature: float = Field(
        default=0.1, ge=0.0, le=2.0, description="Temperature for generation"
    )
    max_tokens: int = Field(default=2000, gt=0, description="Maximum tokens to generate")
    api_key: str | None = Field(default=None, description="API key (supports env:VAR_NAME syntax)")
    base_url: str | None = Field(
        default=None, description="Base URL for the API (e.g., for Ollama)"
    )

    @field_validator("api_key", mode="before")
    @classmethod
    def resolve_env_var(cls, v: str | None) -> str | None:
        """Resolve env:VAR_NAME syntax to actual environment variable value."""
        if v is None:
            return None
        if v.startswith("env:"):
            env_var = v[4:]
            return os.environ.get(env_var)
        return v


class LLMProvider(BaseModel):
    """LLM provider configuration."""

    provider: LLMProviderType = Field(default=LLMProviderType.OPENAI, description="LLM provider")
    config: LLMConfig = Field(
        default_factory=LLMConfig, description="Provider-specific configuration"
    )


# =============================================================================
# Embedder Providers
# =============================================================================


class EmbedderProviderType(str, Enum):
    """Supported embedder providers."""

    OPENAI = "openai"
    AZURE_OPENAI = "azure_openai"
    OLLAMA = "ollama"
    HUGGINGFACE = "huggingface"
    VERTEXAI = "vertexai"
    GEMINI = "gemini"
    LMSTUDIO = "lmstudio"
    TOGETHER = "together"
    AWS_BEDROCK = "aws_bedrock"


class EmbedderConfig(BaseModel):
    """Embedder configuration settings."""

    model: str = Field(default="text-embedding-3-small", description="Embedding model name")
    api_key: str | None = Field(default=None, description="API key (supports env:VAR_NAME syntax)")
    base_url: str | None = Field(
        default=None, description="Base URL for the API (e.g., for Ollama)"
    )
    embedding_dims: int | None = Field(
        default=None, description="Embedding dimensions (auto-detected if not set)"
    )

    @field_validator("api_key", mode="before")
    @classmethod
    def resolve_env_var(cls, v: str | None) -> str | None:
        """Resolve env:VAR_NAME syntax to actual environment variable value."""
        if v is None:
            return None
        if v.startswith("env:"):
            env_var = v[4:]
            return os.environ.get(env_var)
        return v


class EmbedderProvider(BaseModel):
    """Embedder provider configuration."""

    provider: EmbedderProviderType = Field(
        default=EmbedderProviderType.OPENAI, description="Embedder provider"
    )
    config: EmbedderConfig = Field(
        default_factory=EmbedderConfig, description="Provider-specific configuration"
    )


# =============================================================================
# Vector Store Providers
# =============================================================================


class VectorStoreProviderType(str, Enum):
    """Supported vector store providers."""

    QDRANT = "qdrant"
    CHROMA = "chroma"
    PINECONE = "pinecone"
    MILVUS = "milvus"
    WEAVIATE = "weaviate"
    PGVECTOR = "pgvector"
    FAISS = "faiss"
    REDIS = "redis"
    AZURE_AI_SEARCH = "azure_ai_search"
    VERTEX_AI_VECTOR_SEARCH = "vertex_ai_vector_search"
    MONGODB_ATLAS = "mongodb_atlas"
    MEMORY = "memory"  # In-memory for testing


class VectorStoreConfig(BaseModel):
    """Vector store configuration settings.

    Note: Different providers require different configuration options.
    Common fields are defined here, and additional provider-specific
    config can be passed through the extra dict.
    """

    # Common settings
    collection_name: str = Field(default="mem0_memories", description="Collection/index name")

    # Qdrant/Milvus/Chroma settings
    host: str | None = Field(default=None, description="Server host (e.g., localhost)")
    port: int | None = Field(default=None, description="Server port (e.g., 6333 for Qdrant)")
    path: str | None = Field(default=None, description="Local storage path for embedded databases")

    # Cloud service settings
    api_key: str | None = Field(default=None, description="API key for cloud services")
    url: str | None = Field(default=None, description="Full URL for cloud services")

    # Embedding dimensions
    embedding_model_dims: int | None = Field(default=None, description="Embedding dimensions")

    # Extra provider-specific config
    extra: dict[str, Any] = Field(
        default_factory=dict, description="Additional provider-specific settings"
    )

    @field_validator("api_key", mode="before")
    @classmethod
    def resolve_env_var(cls, v: str | None) -> str | None:
        """Resolve env:VAR_NAME syntax to actual environment variable value."""
        if v is None:
            return None
        if v.startswith("env:"):
            env_var = v[4:]
            return os.environ.get(env_var)
        return v


class VectorStoreProvider(BaseModel):
    """Vector store provider configuration."""

    provider: VectorStoreProviderType = Field(
        default=VectorStoreProviderType.QDRANT, description="Vector store provider"
    )
    config: VectorStoreConfig = Field(
        default_factory=VectorStoreConfig, description="Provider-specific configuration"
    )


# =============================================================================
# Graph Store Providers
# =============================================================================


class GraphStoreProviderType(str, Enum):
    """Supported graph store providers."""

    NEO4J = "neo4j"
    MEMGRAPH = "memgraph"
    NEPTUNE = "neptune"
    KUZU = "kuzu"


class GraphStoreConfig(BaseModel):
    """Graph store configuration settings."""

    url: str | None = Field(default=None, description="Graph database URL")
    username: str | None = Field(default=None, description="Username for authentication")
    password: str | None = Field(default=None, description="Password for authentication")
    database: str | None = Field(default=None, description="Database name (Neo4j)")
    db: str | None = Field(default=None, description="Database path (Kuzu, default: :memory:)")
    endpoint: str | None = Field(default=None, description="Neptune endpoint")
    base_label: bool | None = Field(default=None, description="Use base node label __Entity__")

    @field_validator("password", mode="before")
    @classmethod
    def resolve_env_var(cls, v: str | None) -> str | None:
        """Resolve env:VAR_NAME syntax to actual environment variable value."""
        if v is None:
            return None
        if v.startswith("env:"):
            env_var = v[4:]
            return os.environ.get(env_var)
        return v


class GraphStoreLLMConfig(BaseModel):
    """LLM configuration for graph store queries."""

    provider: LLMProviderType = Field(default=LLMProviderType.OPENAI, description="LLM provider")
    config: LLMConfig = Field(default_factory=LLMConfig, description="LLM configuration")


class GraphStoreProvider(BaseModel):
    """Graph store provider configuration."""

    provider: GraphStoreProviderType = Field(
        default=GraphStoreProviderType.NEO4J, description="Graph store provider"
    )
    config: GraphStoreConfig = Field(
        default_factory=GraphStoreConfig, description="Provider-specific configuration"
    )
    llm: GraphStoreLLMConfig | None = Field(
        default=None, description="LLM for graph store queries (uses main LLM if not set)"
    )
    custom_prompt: str | None = Field(
        default=None, description="Custom prompt for entity extraction"
    )
    threshold: float = Field(
        default=0.7, ge=0.0, le=1.0, description="Embedding similarity threshold for node matching"
    )


# =============================================================================
# Custom Prompts Settings
# =============================================================================


class CustomPromptsConfig(BaseModel):
    """Custom prompts for memory extraction and management."""

    fact_extraction: str | None = Field(
        default=None,
        description="Custom prompt for fact extraction from conversations. "
        "Should instruct the LLM to extract facts and return JSON with 'facts' key.",
    )
    update_memory: str | None = Field(
        default=None, description="Custom prompt for memory update decisions."
    )


# =============================================================================
# OpenMemory Settings
# =============================================================================


class OpenMemoryConfig(BaseModel):
    """OpenMemory-specific configuration."""

    custom_instructions: str | None = Field(
        default=None, description="Custom instructions for memory extraction and management"
    )
    custom_categories: dict[str, str] | None = Field(
        default=None, description="Custom category definitions for memory tagging"
    )


# =============================================================================
# Server Settings
# =============================================================================


class ServerConfig(BaseModel):
    """Server configuration settings."""

    host: str = Field(default="0.0.0.0", description="Server host to bind to")
    port: int = Field(default=8765, ge=1, le=65535, description="Server port")
    user_id: str = Field(default="default", description="Default user ID for memories")
    reload: bool = Field(default=False, description="Enable auto-reload for development")
    log_level: Literal["debug", "info", "warning", "error"] = Field(
        default="info", description="Logging level"
    )
    performance_logging: bool = Field(
        default=False,
        description="Enable per-request performance logging (tool name, duration, status)",
    )


# =============================================================================
# Main Configuration
# =============================================================================


class Mem0ServerConfig(BaseModel):
    """Complete mem0-server configuration."""

    server: ServerConfig = Field(default_factory=ServerConfig, description="Server settings")
    llm: LLMProvider = Field(default_factory=LLMProvider, description="LLM provider configuration")
    embedder: EmbedderProvider = Field(
        default_factory=EmbedderProvider, description="Embedder provider configuration"
    )
    vector_store: VectorStoreProvider = Field(
        default_factory=VectorStoreProvider, description="Vector store configuration"
    )
    openmemory: OpenMemoryConfig = Field(
        default_factory=OpenMemoryConfig, description="OpenMemory-specific settings"
    )
    graph_store: GraphStoreProvider | None = Field(
        default=None, description="Graph store configuration (optional, enables knowledge graph)"
    )
    custom_prompts: CustomPromptsConfig = Field(
        default_factory=CustomPromptsConfig,
        description="Custom prompts for fact extraction and memory updates",
    )
    vector_store: VectorStoreProvider = Field(
        default_factory=VectorStoreProvider, description="Vector store configuration"
    )
    openmemory: OpenMemoryConfig = Field(
        default_factory=OpenMemoryConfig, description="OpenMemory-specific settings"
    )
    graph_store: GraphStoreProvider | None = Field(
        default=None, description="Graph store configuration (optional, enables knowledge graph)"
    )

    def to_mem0_config(self) -> dict[str, Any]:
        """Convert to mem0 library configuration format."""
        config: dict[str, Any] = {
            "version": "v1.1",
        }

        # LLM config
        llm_config: dict[str, Any] = {
            "model": self.llm.config.model,
            "temperature": self.llm.config.temperature,
            "max_tokens": self.llm.config.max_tokens,
        }
        if self.llm.config.api_key:
            llm_config["api_key"] = self.llm.config.api_key
        if self.llm.config.base_url:
            if self.llm.provider == LLMProviderType.OLLAMA:
                llm_config["ollama_base_url"] = self.llm.config.base_url
            else:
                llm_config["openai_base_url"] = self.llm.config.base_url
                if not self.llm.config.api_key:
                    llm_config["api_key"] = "lm-studio"

        config["llm"] = {
            "provider": self.llm.provider.value,
            "config": llm_config,
        }

        # Embedder config
        embedder_config: dict[str, Any] = {
            "model": self.embedder.config.model,
        }
        if self.embedder.config.api_key:
            embedder_config["api_key"] = self.embedder.config.api_key
        if self.embedder.config.base_url:
            if self.embedder.provider == EmbedderProviderType.OLLAMA:
                embedder_config["ollama_base_url"] = self.embedder.config.base_url
            else:
                embedder_config["openai_base_url"] = self.embedder.config.base_url
                if not self.embedder.config.api_key:
                    embedder_config["api_key"] = "lm-studio"
        if self.embedder.config.embedding_dims:
            embedder_config["embedding_dims"] = self.embedder.config.embedding_dims

        config["embedder"] = {
            "provider": self.embedder.provider.value,
            "config": embedder_config,
        }

        # Vector store config
        vs_config: dict[str, Any] = {
            "collection_name": self.vector_store.config.collection_name,
        }
        if self.vector_store.config.host:
            vs_config["host"] = self.vector_store.config.host
        if self.vector_store.config.port:
            vs_config["port"] = self.vector_store.config.port
        if self.vector_store.config.path:
            vs_config["path"] = self.vector_store.config.path
        if self.vector_store.config.api_key:
            vs_config["api_key"] = self.vector_store.config.api_key
        if self.vector_store.config.url:
            vs_config["url"] = self.vector_store.config.url
        if self.vector_store.config.embedding_model_dims:
            vs_config["embedding_model_dims"] = self.vector_store.config.embedding_model_dims
        # Merge extra config
        vs_config.update(self.vector_store.config.extra)

        config["vector_store"] = {
            "provider": self.vector_store.provider.value,
            "config": vs_config,
        }

        # Custom instructions (if any) - legacy support
        if self.openmemory.custom_instructions:
            config["custom_prompt"] = self.openmemory.custom_instructions

        # Custom prompts for fact extraction and memory updates
        if self.custom_prompts.fact_extraction:
            config["custom_fact_extraction_prompt"] = self.custom_prompts.fact_extraction
        if self.custom_prompts.update_memory:
            config["custom_update_memory_prompt"] = self.custom_prompts.update_memory

        # Graph store config (if enabled)
        if self.graph_store:
            gs_config: dict[str, Any] = {}
            if self.graph_store.config.url:
                gs_config["url"] = self.graph_store.config.url
            if self.graph_store.config.username:
                gs_config["username"] = self.graph_store.config.username
            if self.graph_store.config.password:
                gs_config["password"] = self.graph_store.config.password
            if self.graph_store.config.database:
                gs_config["database"] = self.graph_store.config.database
            if self.graph_store.config.db:
                gs_config["db"] = self.graph_store.config.db
            if self.graph_store.config.endpoint:
                gs_config["endpoint"] = self.graph_store.config.endpoint
            if self.graph_store.config.base_label is not None:
                gs_config["base_label"] = self.graph_store.config.base_label

            graph_store_dict: dict[str, Any] = {
                "provider": self.graph_store.provider.value,
                "config": gs_config,
                "threshold": self.graph_store.threshold,
            }

            if self.graph_store.llm:
                llm_conf: dict[str, Any] = {
                    "model": self.graph_store.llm.config.model,
                    "temperature": self.graph_store.llm.config.temperature,
                    "max_tokens": self.graph_store.llm.config.max_tokens,
                }
                if self.graph_store.llm.config.api_key:
                    llm_conf["api_key"] = self.graph_store.llm.config.api_key
                if self.graph_store.llm.config.base_url:
                    if self.graph_store.llm.provider == LLMProviderType.OLLAMA:
                        llm_conf["ollama_base_url"] = self.graph_store.llm.config.base_url
                    else:
                        llm_conf["openai_base_url"] = self.graph_store.llm.config.base_url
                graph_store_dict["llm"] = {
                    "provider": self.graph_store.llm.provider.value,
                    "config": llm_conf,
                }

            if self.graph_store.custom_prompt:
                graph_store_dict["custom_prompt"] = self.graph_store.custom_prompt

            config["graph_store"] = graph_store_dict

        return config


def get_default_config() -> Mem0ServerConfig:
    """Get default configuration with sensible defaults."""
    return Mem0ServerConfig(
        llm=LLMProvider(
            provider=LLMProviderType.OPENAI,
            config=LLMConfig(
                model="gpt-4o-mini",
                temperature=0.1,
                max_tokens=2000,
                api_key="env:OPENAI_API_KEY",
            ),
        ),
        embedder=EmbedderProvider(
            provider=EmbedderProviderType.OPENAI,
            config=EmbedderConfig(
                model="text-embedding-3-small",
                api_key="env:OPENAI_API_KEY",
                embedding_dims=1536,
            ),
        ),
        vector_store=VectorStoreProvider(
            provider=VectorStoreProviderType.QDRANT,
            config=VectorStoreConfig(
                collection_name="mem0_memories",
                host="localhost",
                port=6333,
                embedding_model_dims=1536,
            ),
        ),
    )



"""
REST API routes for mem0-server.

Provides endpoints for:
- Configuration management (CRUD)
- Memory operations (list, search, get, delete)
- Server status and health
"""

from __future__ import annotations

import logging
from typing import Any

from fastapi import APIRouter, HTTPException, Query
from pydantic import BaseModel, Field

from mem0_server.config import (
    ConfigLoader,
    EmbedderConfig,
    EmbedderProvider,
    EmbedderProviderType,
    LLMConfig,
    LLMProvider,
    LLMProviderType,
    Mem0ServerConfig,
    OpenMemoryConfig,
    VectorStoreConfig,
    VectorStoreProvider,
    VectorStoreProviderType,
)
from mem0_server.config.schema import ServerConfig

logger = logging.getLogger(__name__)

# =============================================================================
# Request/Response Models
# =============================================================================

class ConfigResponse(BaseModel):
    """Configuration response model."""
    server: ServerConfig
    llm: LLMProvider
    embedder: EmbedderProvider
    vector_store: VectorStoreProvider
    openmemory: OpenMemoryConfig
    config_path: str | None = None


class ConfigUpdateRequest(BaseModel):
    """Configuration update request (partial update)."""
    server: ServerConfig | None = None
    llm: LLMProvider | None = None
    embedder: EmbedderProvider | None = None
    vector_store: VectorStoreProvider | None = None
    openmemory: OpenMemoryConfig | None = None


class LLMConfigUpdateRequest(BaseModel):
    """LLM configuration update request."""
    provider: LLMProviderType | None = None
    config: LLMConfig | None = None


class EmbedderConfigUpdateRequest(BaseModel):
    """Embedder configuration update request."""
    provider: EmbedderProviderType | None = None
    config: EmbedderConfig | None = None


class VectorStoreConfigUpdateRequest(BaseModel):
    """Vector store configuration update request."""
    provider: VectorStoreProviderType | None = None
    config: VectorStoreConfig | None = None


class MemoryItem(BaseModel):
    """Memory item model."""
    id: str
    memory: str
    user_id: str | None = None
    hash: str | None = None
    metadata: dict[str, Any] | None = None
    created_at: str | None = None
    updated_at: str | None = None


class MemoryListResponse(BaseModel):
    """Memory list response."""
    results: list[MemoryItem]
    count: int
    user_id: str


class MemorySearchRequest(BaseModel):
    """Memory search request."""
    query: str
    limit: int = Field(default=10, ge=1, le=100)
    user_id: str | None = None


class MemorySearchResponse(BaseModel):
    """Memory search response."""
    results: list[MemoryItem]
    count: int
    query: str


class MemoryAddRequest(BaseModel):
    """Memory add request."""
    text: str
    user_id: str | None = None
    metadata: dict[str, Any] | None = None


class MemoryAddResponse(BaseModel):
    """Memory add response."""
    results: list[dict[str, Any]]
    count: int


class MemoryDeleteRequest(BaseModel):
    """Memory delete request."""
    memory_ids: list[str]


class MemoryDeleteResponse(BaseModel):
    """Memory delete response."""
    deleted: list[str]
    count: int


class StatusResponse(BaseModel):
    """Server status response."""
    status: str
    version: str
    config_loaded: bool
    config_path: str | None = None
    memory_available: bool
    user_id: str
    providers: dict[str, str]


class HealthResponse(BaseModel):
    """Health check response."""
    status: str
    service: str


class ProvidersResponse(BaseModel):
    """Available providers response."""
    llm_providers: list[str]
    embedder_providers: list[str]
    vector_store_providers: list[str]


# =============================================================================
# API Router Factory
# =============================================================================

def create_api_router(
    config: Mem0ServerConfig,
    config_loader: ConfigLoader,
    get_memory_client_func,
) -> APIRouter:
    """Create the API router with all endpoints.
    
    Args:
        config: The server configuration.
        config_loader: The config loader for saving changes.
        get_memory_client_func: Function to get the memory client.
    
    Returns:
        FastAPI APIRouter with all endpoints configured.
    """
    router = APIRouter(prefix="/api/v1", tags=["api"])
    
    # Store mutable config reference
    current_config = {"config": config}
    
    # =========================================================================
    # Health & Status
    # =========================================================================
    
    @router.get("/health", response_model=HealthResponse)
    async def health_check():
        """Health check endpoint."""
        return HealthResponse(status="ok", service="mem0-server")
    
    @router.get("/status", response_model=StatusResponse)
    async def get_status():
        """Get server status and configuration summary."""
        from mem0_server import __version__
        
        cfg = current_config["config"]
        memory_client = await get_memory_client_func()
        
        return StatusResponse(
            status="running",
            version=__version__,
            config_loaded=config_loader.config_path is not None,
            config_path=str(config_loader.config_path) if config_loader.config_path else None,
            memory_available=memory_client is not None,
            user_id=cfg.server.user_id,
            providers={
                "llm": cfg.llm.provider.value,
                "embedder": cfg.embedder.provider.value,
                "vector_store": cfg.vector_store.provider.value,
            },
        )
    
    @router.get("/providers", response_model=ProvidersResponse)
    async def get_available_providers():
        """Get list of all available providers."""
        return ProvidersResponse(
            llm_providers=[p.value for p in LLMProviderType],
            embedder_providers=[p.value for p in EmbedderProviderType],
            vector_store_providers=[p.value for p in VectorStoreProviderType],
        )
    
    # =========================================================================
    # Configuration Management
    # =========================================================================
    
    @router.get("/config", response_model=ConfigResponse)
    async def get_configuration():
        """Get the current configuration."""
        cfg = current_config["config"]
        return ConfigResponse(
            server=cfg.server,
            llm=cfg.llm,
            embedder=cfg.embedder,
            vector_store=cfg.vector_store,
            openmemory=cfg.openmemory,
            config_path=str(config_loader.config_path) if config_loader.config_path else None,
        )
    
    @router.put("/config", response_model=ConfigResponse)
    async def update_configuration(update: ConfigUpdateRequest):
        """Update the entire configuration."""
        cfg = current_config["config"]
        
        # Apply updates
        if update.server:
            cfg.server = update.server
        if update.llm:
            cfg.llm = update.llm
        if update.embedder:
            cfg.embedder = update.embedder
        if update.vector_store:
            cfg.vector_store = update.vector_store
        if update.openmemory:
            cfg.openmemory = update.openmemory
        
        # Save to file if we have a config path
        if config_loader.config_path:
            try:
                config_loader.save(cfg, config_loader.config_path)
            except Exception as e:
                logger.warning(f"Failed to save config: {e}")
        
        current_config["config"] = cfg
        
        return ConfigResponse(
            server=cfg.server,
            llm=cfg.llm,
            embedder=cfg.embedder,
            vector_store=cfg.vector_store,
            openmemory=cfg.openmemory,
            config_path=str(config_loader.config_path) if config_loader.config_path else None,
        )
    
    @router.patch("/config", response_model=ConfigResponse)
    async def patch_configuration(update: ConfigUpdateRequest):
        """Partially update the configuration."""
        return await update_configuration(update)
    
    @router.post("/config/reset", response_model=ConfigResponse)
    async def reset_configuration():
        """Reset configuration to defaults."""
        from mem0_server.config.schema import get_default_config
        
        cfg = get_default_config()
        current_config["config"] = cfg
        
        # Save to file if we have a config path
        if config_loader.config_path:
            try:
                config_loader.save(cfg, config_loader.config_path)
            except Exception as e:
                logger.warning(f"Failed to save config: {e}")
        
        return ConfigResponse(
            server=cfg.server,
            llm=cfg.llm,
            embedder=cfg.embedder,
            vector_store=cfg.vector_store,
            openmemory=cfg.openmemory,
            config_path=str(config_loader.config_path) if config_loader.config_path else None,
        )
    
    # Individual config section endpoints
    @router.get("/config/llm", response_model=LLMProvider)
    async def get_llm_config():
        """Get LLM configuration."""
        return current_config["config"].llm
    
    @router.put("/config/llm", response_model=LLMProvider)
    async def update_llm_config(update: LLMConfigUpdateRequest):
        """Update LLM configuration."""
        cfg = current_config["config"]
        
        if update.provider:
            cfg.llm.provider = update.provider
        if update.config:
            cfg.llm.config = update.config
        
        if config_loader.config_path:
            config_loader.save(cfg, config_loader.config_path)
        
        return cfg.llm
    
    @router.get("/config/embedder", response_model=EmbedderProvider)
    async def get_embedder_config():
        """Get embedder configuration."""
        return current_config["config"].embedder
    
    @router.put("/config/embedder", response_model=EmbedderProvider)
    async def update_embedder_config(update: EmbedderConfigUpdateRequest):
        """Update embedder configuration."""
        cfg = current_config["config"]
        
        if update.provider:
            cfg.embedder.provider = update.provider
        if update.config:
            cfg.embedder.config = update.config
        
        if config_loader.config_path:
            config_loader.save(cfg, config_loader.config_path)
        
        return cfg.embedder
    
    @router.get("/config/vector_store", response_model=VectorStoreProvider)
    async def get_vector_store_config():
        """Get vector store configuration."""
        return current_config["config"].vector_store
    
    @router.put("/config/vector_store", response_model=VectorStoreProvider)
    async def update_vector_store_config(update: VectorStoreConfigUpdateRequest):
        """Update vector store configuration."""
        cfg = current_config["config"]
        
        if update.provider:
            cfg.vector_store.provider = update.provider
        if update.config:
            cfg.vector_store.config = update.config
        
        if config_loader.config_path:
            config_loader.save(cfg, config_loader.config_path)
        
        return cfg.vector_store
    
    @router.get("/config/openmemory", response_model=OpenMemoryConfig)
    async def get_openmemory_config():
        """Get OpenMemory configuration."""
        return current_config["config"].openmemory
    
    @router.put("/config/openmemory", response_model=OpenMemoryConfig)
    async def update_openmemory_config(update: OpenMemoryConfig):
        """Update OpenMemory configuration."""
        cfg = current_config["config"]
        cfg.openmemory = update
        
        if config_loader.config_path:
            config_loader.save(cfg, config_loader.config_path)
        
        return cfg.openmemory
    
    # =========================================================================
    # Memory Operations
    # =========================================================================
    
    @router.get("/memories", response_model=MemoryListResponse)
    async def list_memories(
        user_id: str | None = Query(None, description="User ID to filter memories"),
        limit: int = Query(100, ge=1, le=1000, description="Maximum number of memories to return"),
    ):
        """List all memories for a user."""
        cfg = current_config["config"]
        uid = user_id or cfg.server.user_id
        
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            memories = await memory_client.get_all(user_id=uid)
            
            results = []
            if isinstance(memories, dict) and "results" in memories:
                for m in memories["results"][:limit]:
                    results.append(MemoryItem(
                        id=m.get("id", ""),
                        memory=m.get("memory", ""),
                        user_id=m.get("user_id"),
                        hash=m.get("hash"),
                        metadata=m.get("metadata"),
                        created_at=m.get("created_at"),
                        updated_at=m.get("updated_at"),
                    ))
            elif isinstance(memories, list):
                for m in memories[:limit]:
                    results.append(MemoryItem(
                        id=m.get("id", ""),
                        memory=m.get("memory", ""),
                        user_id=m.get("user_id"),
                        hash=m.get("hash"),
                        metadata=m.get("metadata"),
                        created_at=m.get("created_at"),
                        updated_at=m.get("updated_at"),
                    ))
            
            return MemoryListResponse(
                results=results,
                count=len(results),
                user_id=uid,
            )
        except Exception as e:
            logger.exception(f"Error listing memories: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.post("/memories/search", response_model=MemorySearchResponse)
    async def search_memories(request: MemorySearchRequest):
        """Search memories by query."""
        cfg = current_config["config"]
        uid = request.user_id or cfg.server.user_id
        
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            results_raw = await memory_client.search(
                query=request.query,
                user_id=uid,
                limit=request.limit,
            )
            
            results = []
            items = results_raw.get("results", []) if isinstance(results_raw, dict) else results_raw
            
            for m in items:
                results.append(MemoryItem(
                    id=m.get("id", ""),
                    memory=m.get("memory", ""),
                    user_id=m.get("user_id"),
                    hash=m.get("hash"),
                    metadata=m.get("metadata"),
                    created_at=m.get("created_at"),
                    updated_at=m.get("updated_at"),
                ))
            
            return MemorySearchResponse(
                results=results,
                count=len(results),
                query=request.query,
            )
        except Exception as e:
            logger.exception(f"Error searching memories: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.get("/memories/{memory_id}", response_model=MemoryItem)
    async def get_memory(memory_id: str):
        """Get a specific memory by ID."""
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            memory = await memory_client.get(memory_id)
            
            if not memory:
                raise HTTPException(status_code=404, detail="Memory not found")
            
            return MemoryItem(
                id=memory.get("id", memory_id),
                memory=memory.get("memory", ""),
                user_id=memory.get("user_id"),
                hash=memory.get("hash"),
                metadata=memory.get("metadata"),
                created_at=memory.get("created_at"),
                updated_at=memory.get("updated_at"),
            )
        except HTTPException:
            raise
        except Exception as e:
            logger.exception(f"Error getting memory: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.post("/memories", response_model=MemoryAddResponse)
    async def add_memory(request: MemoryAddRequest):
        """Add a new memory."""
        cfg = current_config["config"]
        uid = request.user_id or cfg.server.user_id
        
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            result = await memory_client.add(
                request.text,
                user_id=uid,
                metadata=request.metadata or {},
            )
            
            results = result.get("results", []) if isinstance(result, dict) else []
            
            return MemoryAddResponse(
                results=results,
                count=len(results),
            )
        except Exception as e:
            logger.exception(f"Error adding memory: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.delete("/memories/{memory_id}")
    async def delete_memory(memory_id: str):
        """Delete a specific memory."""
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            await memory_client.delete(memory_id)
            return {"deleted": memory_id, "status": "ok"}
        except Exception as e:
            logger.exception(f"Error deleting memory: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.post("/memories/delete", response_model=MemoryDeleteResponse)
    async def delete_memories(request: MemoryDeleteRequest):
        """Delete multiple memories by IDs."""
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        deleted = []
        for memory_id in request.memory_ids:
            try:
                await memory_client.delete(memory_id)
                deleted.append(memory_id)
            except Exception as e:
                logger.warning(f"Failed to delete memory {memory_id}: {e}")
        
        return MemoryDeleteResponse(
            deleted=deleted,
            count=len(deleted),
        )
    
    @router.delete("/memories")
    async def delete_all_memories(
        user_id: str | None = Query(None, description="User ID to delete memories for"),
        confirm: bool = Query(False, description="Confirm deletion"),
    ):
        """Delete all memories for a user."""
        if not confirm:
            raise HTTPException(
                status_code=400, 
                detail="Set confirm=true to delete all memories"
            )
        
        cfg = current_config["config"]
        uid = user_id or cfg.server.user_id
        
        memory_client = await get_memory_client_func()
        if not memory_client:
            raise HTTPException(status_code=503, detail="Memory system unavailable")
        
        try:
            await memory_client.delete_all(user_id=uid)
            return {"status": "ok", "message": f"All memories deleted for user {uid}"}
        except Exception as e:
            logger.exception(f"Error deleting all memories: {e}")
            raise HTTPException(status_code=500, detail=str(e)) from None
    
    @router.get("/memories/stats/summary")
    async def get_memory_stats(
        user_id: str | None = Query(None, description="User ID to get stats for"),
    ):
        """Get memory statistics."""
        cfg = current_config["config"]
        uid = user_id or cfg.server.user_id
        
        memory_client = await get_memory_client_func()
        if not memory_client:
            return {
                "status": "unavailable",
                "message": "Memory system not initialized",
                "user_id": uid,
                "memory_count": 0,
            }
        
        try:
            memories = await memory_client.get_all(user_id=uid)
            
            if isinstance(memories, dict) and "results" in memories:
                count = len(memories["results"])
            elif isinstance(memories, list):
                count = len(memories)
            else:
                count = 0
            
            return {
                "status": "ok",
                "user_id": uid,
                "memory_count": count,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
                "user_id": uid,
                "memory_count": 0,
            }
    
    return router



"""Configuration management for mem0-server."""

from mem0_server.config.loader import ConfigLoader
from mem0_server.config.schema import (
    EmbedderConfig,
    EmbedderProvider,
    EmbedderProviderType,
    GraphStoreConfig,
    GraphStoreLLMConfig,
    GraphStoreProvider,
    GraphStoreProviderType,
    LLMConfig,
    LLMProvider,
    LLMProviderType,
    Mem0ServerConfig,
    OpenMemoryConfig,
    VectorStoreConfig,
    VectorStoreProvider,
    VectorStoreProviderType,
    get_default_config,
)

__all__ = [
    "Mem0ServerConfig",
    "LLMProvider",
    "LLMProviderType",
    "LLMConfig",
    "EmbedderProvider",
    "EmbedderProviderType",
    "EmbedderConfig",
    "VectorStoreProvider",
    "VectorStoreProviderType",
    "VectorStoreConfig",
    "GraphStoreProvider",
    "GraphStoreProviderType",
    "GraphStoreConfig",
    "GraphStoreLLMConfig",
    "OpenMemoryConfig",
    "ConfigLoader",
    "get_default_config",
]



"""Configuration file loader for mem0-server.

Supports loading configuration from:
- YAML files
- JSON files
- Environment variables
- Interactive prompts
"""

from __future__ import annotations

import json
import logging
import os
import re
from pathlib import Path
from typing import Any

import yaml
from pydantic import ValidationError

from mem0_server.config.schema import Mem0ServerConfig, get_default_config

logger = logging.getLogger(__name__)

MEM0_HOME = Path.home() / ".mem0-open-mcp"
MEM0_LOGS_DIR = MEM0_HOME / "logs"


class ConfigLoader:
    """Load and save mem0-server configuration from various sources."""

    DEFAULT_CONFIG_PATHS = [
        MEM0_HOME / "mem0-open-mcp.yaml",
        MEM0_HOME / "mem0-open-mcp.yml",
        MEM0_HOME / "mem0-open-mcp.json",
        Path.home() / ".config" / "mem0-open-mcp.yaml",
        Path.home() / ".config" / "mem0-open-mcp.yml",
        Path.home() / ".config" / "mem0-open-mcp.json",
        Path("mem0-open-mcp.yaml"),
        Path("mem0-open-mcp.yml"),
        Path("mem0-open-mcp.json"),
    ]

    def __init__(self, config_path: Path | str | None = None):
        """Initialize the config loader.

        Args:
            config_path: Explicit path to config file. If None, searches default locations.
        """
        self.config_path = Path(config_path) if config_path else None
        self._config: Mem0ServerConfig | None = None

    @property
    def config(self) -> Mem0ServerConfig:
        """Get the current configuration, loading if necessary."""
        if self._config is None:
            self._config = self.load()
        return self._config

    def find_config_file(self) -> Path | None:
        """Find the first existing config file from default paths."""
        if self.config_path and self.config_path.exists():
            return self.config_path

        for path in self.DEFAULT_CONFIG_PATHS:
            if path.exists():
                return path

        return None

    def load(self, path: Path | str | None = None) -> Mem0ServerConfig:
        """Load configuration from file.

        Args:
            path: Path to config file. If None, searches default locations.

        Returns:
            Loaded configuration, or defaults if no config file found.
        """
        config_path = Path(path) if path else self.find_config_file()

        if config_path is None:
            logger.info("No config file found, using defaults")
            return get_default_config()

        logger.info(f"Loading config from: {config_path}")

        try:
            with open(config_path) as f:
                if config_path.suffix in (".yaml", ".yml"):
                    raw_config = yaml.safe_load(f)
                elif config_path.suffix == ".json":
                    raw_config = json.load(f)
                else:
                    raise ValueError(f"Unsupported config file format: {config_path.suffix}")

            if raw_config is None:
                raw_config = {}

            # Resolve environment variables in the config
            resolved_config = self._resolve_env_vars(raw_config)

            # Parse into Pydantic model
            config = Mem0ServerConfig.model_validate(resolved_config)
            self._config = config
            self.config_path = config_path
            return config

        except ValidationError as e:
            logger.error(f"Config validation error: {e}")
            raise
        except Exception as e:
            logger.error(f"Error loading config: {e}")
            raise

    def save(self, config: Mem0ServerConfig, path: Path | str | None = None) -> Path:
        """Save configuration to file.

        Args:
            config: Configuration to save.
            path: Path to save to. If None, uses current config_path or default.

        Returns:
            Path where config was saved.
        """
        save_path = Path(path) if path else (self.config_path or Path("mem0-open-mcp.yaml"))

        # Ensure parent directory exists
        save_path.parent.mkdir(parents=True, exist_ok=True)

        # Convert to dict, preserving env: syntax where appropriate
        config_dict = self._config_to_saveable_dict(config)

        with open(save_path, "w") as f:
            if save_path.suffix in (".yaml", ".yml"):
                yaml.safe_dump(config_dict, f, default_flow_style=False, sort_keys=False)
            elif save_path.suffix == ".json":
                json.dump(config_dict, f, indent=2)
            else:
                # Default to YAML
                yaml.safe_dump(config_dict, f, default_flow_style=False, sort_keys=False)

        logger.info(f"Saved config to: {save_path}")
        self._config = config
        self.config_path = save_path
        return save_path

    def _resolve_env_vars(self, obj: Any) -> Any:
        """Recursively resolve environment variables in config.

        Supports two syntaxes:
        - ${VAR_NAME}: Substitutes with env var value
        - env:VAR_NAME: Kept as-is for Pydantic to resolve later
        """
        if isinstance(obj, str):
            # Handle ${VAR_NAME} syntax
            def replace_env(match: re.Match[str]) -> str:
                var_name = match.group(1)
                value = os.environ.get(var_name, "")
                if not value:
                    logger.warning(f"Environment variable {var_name} is not set")
                return value

            return re.sub(r"\$\{([^}]+)\}", replace_env, obj)

        elif isinstance(obj, dict):
            return {k: self._resolve_env_vars(v) for k, v in obj.items()}

        elif isinstance(obj, list):
            return [self._resolve_env_vars(item) for item in obj]

        return obj

    def _config_to_saveable_dict(self, config: Mem0ServerConfig) -> dict[str, Any]:
        """Convert config to dict suitable for saving.

        Converts API keys back to env: syntax if they look like they came from env vars.
        """
        # Use mode="json" to convert enums to their string values for YAML serialization
        config_dict = config.model_dump(exclude_none=True, exclude_unset=False, mode="json")

        # Replace known API keys with env: syntax for safety
        if "llm" in config_dict and "config" in config_dict["llm"]:
            if "api_key" in config_dict["llm"]["config"]:
                # Check common env var names
                provider = config_dict["llm"].get("provider", "openai")
                env_var = self._get_api_key_env_var(provider)
                if env_var and os.environ.get(env_var):
                    config_dict["llm"]["config"]["api_key"] = f"env:{env_var}"

        if "embedder" in config_dict and "config" in config_dict["embedder"]:
            if "api_key" in config_dict["embedder"]["config"]:
                provider = config_dict["embedder"].get("provider", "openai")
                env_var = self._get_api_key_env_var(provider)
                if env_var and os.environ.get(env_var):
                    config_dict["embedder"]["config"]["api_key"] = f"env:{env_var}"

        return config_dict

    def _get_api_key_env_var(self, provider: str) -> str | None:
        """Get the expected environment variable name for a provider's API key."""
        provider_env_vars = {
            "openai": "OPENAI_API_KEY",
            "anthropic": "ANTHROPIC_API_KEY",
            "azure_openai": "AZURE_OPENAI_API_KEY",
            "together": "TOGETHER_API_KEY",
            "groq": "GROQ_API_KEY",
            "mistralai": "MISTRAL_API_KEY",
            "google_ai": "GOOGLE_API_KEY",
            "gemini": "GOOGLE_API_KEY",
            "deepseek": "DEEPSEEK_API_KEY",
            "xai": "XAI_API_KEY",
        }
        return provider_env_vars.get(provider.lower())

    @classmethod
    def create_default_config_file(cls, path: Path | str | None = None) -> Path:
        """Create a default configuration file.

        Args:
            path: Path to create config at. Defaults to mem0-open-mcp.yaml in current dir.

        Returns:
            Path where config was created.
        """
        save_path = Path(path) if path else Path("mem0-open-mcp.yaml")
        loader = cls()
        config = get_default_config()
        return loader.save(config, save_path)



"""
mem0-server: Standalone MCP server for mem0 with web configuration UI.

This package provides a CLI tool to run mem0 as an MCP server without Docker,
with optional web UI for configuration management.
"""

from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("mem0-open-mcp")
except PackageNotFoundError:
    __version__ = "0.0.0-dev"
